import abjad

import auxjad


def test_enforce_time_signature_01():
    staff = abjad.Staff(r"c'1 d'1")
    time_signatures = abjad.TimeSignature((2, 4))
    auxjad.mutate.enforce_time_signature(staff, time_signatures)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            c'2
            ~
            c'2
            d'2
            ~
            d'2
        }
        """
    )


def test_enforce_time_signature_02():
    staff = abjad.Staff(r"c'1 d'1")
    auxjad.mutate.enforce_time_signature(staff, (3, 4))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2.
            ~
            c'4
            d'2
            ~
            d'2
            r4
        }
        """
    )


def test_enforce_time_signature_03():
    staff = abjad.Staff(r"c'1 d'1 e'1 f'1")
    auxjad.mutate.enforce_time_signature(
        staff,
        abjad.TimeSignature((3, 4)),
        close_container=True,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2.
            ~
            c'4
            d'2
            ~
            d'2
            e'4
            ~
            e'2.
            f'2.
            ~
            \time 1/4
            f'4
        }
        """
    )


def test_enforce_time_signature_04():
    staff = abjad.Staff(r"c'1 d'1 e'1 f'1")
    auxjad.mutate.enforce_time_signature(
        staff,
        abjad.TimeSignature((3, 4)),
        fill_with_rests=False,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2.
            ~
            c'4
            d'2
            ~
            d'2
            e'4
            ~
            e'2.
            f'2.
            ~
            f'4
        }
        """
    )


def test_enforce_time_signature_05():
    staff = abjad.Staff(r"c'1 d'1")
    time_signatures = [abjad.TimeSignature((3, 4)),
                       abjad.TimeSignature((5, 4)),
                       ]
    auxjad.mutate.enforce_time_signature(staff, time_signatures)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2.
            ~
            \time 5/4
            c'4
            d'1
        }
        """
    )


def test_enforce_time_signature_06():
    staff = abjad.Staff(r"c'1 d'1 e'1 f'1")
    time_signatures = [(2, 4),
                       (2, 4),
                       (4, 4),
                       ]
    auxjad.mutate.enforce_time_signature(staff, time_signatures)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            c'2
            ~
            c'2
            \time 4/4
            d'1
            e'1
            f'1
        }
        """
    )


def test_enforce_time_signature_07():
    staff = abjad.Staff(r"c'1 d'1 e'1 f'1")
    time_signatures = [abjad.TimeSignature((3, 8)),
                       abjad.TimeSignature((2, 8)),
                       ]
    auxjad.mutate.enforce_time_signature(
        staff,
        time_signatures,
        cyclic=True,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/8
            c'4.
            ~
            \time 2/8
            c'4
            ~
            \time 3/8
            c'4.
            \time 2/8
            d'4
            ~
            \time 3/8
            d'4.
            ~
            \time 2/8
            d'4
            ~
            \time 3/8
            d'8
            e'4
            ~
            \time 2/8
            e'4
            ~
            \time 3/8
            e'4.
            ~
            \time 2/8
            e'8
            f'8
            ~
            \time 3/8
            f'4.
            ~
            \time 2/8
            f'4
            ~
            \time 3/8
            f'4
            r8
        }
        """
    )


def test_enforce_time_signature_08():
    staff = abjad.Staff(r"\times 2/3 {c'2 d'2 e'2} f'1")
    time_signatures = [abjad.TimeSignature((2, 4)),
                       abjad.TimeSignature((3, 4)),
                       ]
    auxjad.mutate.enforce_time_signature(staff, time_signatures)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                \time 2/4
                c'2
                d'4
                ~
            }
            \times 2/3
            {
                \time 3/4
                d'4
                e'2
            }
            f'4
            ~
            f'2.
        }
        """
    )


def test_enforce_time_signature_09():
    staff = staff = abjad.Staff(r"\time 3/4 c'2. d'2. e'2. f'2.")
    time_signatures = [abjad.TimeSignature((5, 8)),
                       abjad.TimeSignature((1, 16)),
                       abjad.TimeSignature((2, 4)),
                       ]
    auxjad.mutate.enforce_time_signature(
        staff,
        time_signatures,
        cyclic=True,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/8
            c'4.
            ~
            c'4
            ~
            \time 1/16
            c'16
            ~
            \time 2/4
            c'16
            d'4..
            ~
            \time 5/8
            d'4
            ~
            d'16
            e'16
            ~
            e'4
            ~
            \time 1/16
            e'16
            ~
            \time 2/4
            e'4.
            f'8
            ~
            \time 5/8
            f'4.
            ~
            f'4
        }
        """
    )


def test_enforce_time_signature_10():
    staff = abjad.Staff(r"c'1 d'1 e'1 f'1")
    time_signatures = [(2, 4),
                       None,
                       None,
                       (3, 4),
                       None,
                       (4, 4),
                       ]
    auxjad.mutate.enforce_time_signature(staff, time_signatures)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            c'2
            ~
            c'2
            d'2
            ~
            \time 3/4
            d'2
            e'4
            ~
            e'2.
            \time 4/4
            f'1
        }
        """
    )


def test_enforce_time_signature_11():
    staff = abjad.Staff(r"c'1 d'1 e'2. f'2.")
    time_signatures = [None,
                       None,
                       (3, 4),
                       None,
                       ]
    auxjad.mutate.enforce_time_signature(staff, time_signatures)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            d'1
            \time 3/4
            e'2.
            f'2.
        }
        """
    )


def test_enforce_time_signature_12():
    staff = abjad.Staff(r"\time 3/4 d'8. e'16 ~ e'2 ~ e'4.. c'4 d'16")
    auxjad.mutate.enforce_time_signature(staff, abjad.TimeSignature((3, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'8.
            e'16
            ~
            e'2
            ~
            e'4..
            c'16
            ~
            c'8.
            d'16
        }
        """
    )
    staff = abjad.Staff(r"\time 3/4 d'8. e'16 ~ e'2 ~ e'4.. c'4 d'16")
    auxjad.mutate.enforce_time_signature(
        staff,
        abjad.TimeSignature((3, 4)),
        boundary_depth=1,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'8.
            e'16
            ~
            e'2
            ~
            e'4
            ~
            e'8.
            c'16
            ~
            c'8.
            d'16
        }
        """
    )


def test_enforce_time_signature_13():
    staff = abjad.Staff(r"c'1 ~ c'4 r8 d'4. e'4")
    time_signatures = [abjad.TimeSignature((5, 4)),
                       abjad.TimeSignature((3, 4)),
                       ]
    auxjad.mutate.enforce_time_signature(staff, time_signatures)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/4
            c'2.
            ~
            c'2
            \time 3/4
            r8
            d'4.
            e'4
        }
        """
    )
    staff = abjad.Staff(r"c'1 ~ c'4 r8 d'4. e'4")
    time_signatures = [abjad.TimeSignature((5, 4)),
                       abjad.TimeSignature((3, 4)),
                       ]
    auxjad.mutate.enforce_time_signature(
        staff,
        time_signatures,
        disable_rewrite_meter=True,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/4
            c'1
            ~
            c'4
            \time 3/4
            r8
            d'4.
            e'4
        }
        """
    )


def test_enforce_time_signature_14():
    staff = abjad.Staff(r"c'4. d'8 e'2")
    auxjad.mutate.enforce_time_signature(staff, abjad.TimeSignature((4, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4.
            d'8
            e'2
        }
        """
    )
    staff = abjad.Staff(r"c'4. d'8 e'2")
    auxjad.mutate.enforce_time_signature(
        staff,
        abjad.TimeSignature((4, 4)),
        boundary_depth=1,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            ~
            c'8
            d'8
            e'2
        }
        """
    )


def test_enforce_time_signature_15():
    staff = abjad.Staff(r"\time 3/4 d'8. e'16 ~ e'2 ~ e'4.. c'4 d'16")
    auxjad.mutate.enforce_time_signature(staff, abjad.TimeSignature((3, 4)))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'8.
            e'16
            ~
            e'2
            ~
            e'4..
            c'16
            ~
            c'8.
            d'16
        }
        """
    )
    staff = abjad.Staff(r"\time 3/4 d'8. e'16 ~ e'2 ~ e'4.. c'4 d'16")
    auxjad.mutate.enforce_time_signature(
        staff,
        abjad.TimeSignature((3, 4)),
        maximum_dot_count=1,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'8.
            e'16
            ~
            e'2
            ~
            e'4
            ~
            e'8.
            c'16
            ~
            c'8.
            d'16
        }
        """
    )


def test_enforce_time_signature_16():
    staff = abjad.Staff(r"\times 6/7 {c'4. r16} \times 6/7 {d'4. r16}")
    auxjad.mutate.enforce_time_signature(
        staff,
        abjad.TimeSignature((3, 4)),
        boundary_depth=1,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \tweak text #tuplet-number::calc-fraction-text
            \times 6/7
            {
                \time 3/4
                c'8.
                ~
                c'8
                ~
                c'16
                r16
            }
            \tweak text #tuplet-number::calc-fraction-text
            \times 6/7
            {
                d'8.
                ~
                d'8
                ~
                d'16
                r16
            }
        }
        """
    )
    staff = abjad.Staff(r"\times 6/7 {c'4. r16} \times 6/7 {d'4. r16}")
    auxjad.mutate.enforce_time_signature(
        staff,
        abjad.TimeSignature((3, 4)),
        boundary_depth=1,
        rewrite_tuplets=False,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \tweak text #tuplet-number::calc-fraction-text
            \times 6/7
            {
                \time 3/4
                c'4.
                r16
            }
            \tweak text #tuplet-number::calc-fraction-text
            \times 6/7
            {
                d'4.
                r16
            }
        }
        """
    )



import abjad
import pytest

import auxjad


def test_sustain_notes_01():
    staff = abjad.Staff(r"c'16 r8. d'16 r8. e'16 r8. f'16 r8.")
    auxjad.mutate.sustain_notes(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_sustain_notes_02():
    staff = abjad.Staff(r"c'16 r8. c'16 r8. c'16 r8. c'16 r8.")
    auxjad.mutate.sustain_notes(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            c'4
            c'4
            c'4
        }
        """
    )


def test_sustain_notes_03():
    staff = abjad.Staff(r"<c' e'>16 r8. <c' e'>4 <c' e'>4 <c' e'>16 r8.")
    auxjad.mutate.sustain_notes(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <c' e'>4
            <c' e'>4
            <c' e'>4
            <c' e'>4
        }
        """
    )


def test_sustain_notes_04():
    staff = abjad.Staff(r"\times 2/3 {c'4 d'4 r4} r8 e'8 \times 2/3 {f'8 r4}")
    auxjad.mutate.sustain_notes(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                c'4
                d'2
                ~
            }
            d'8
            e'8
            f'4
        }
        """
    )


def test_sustain_notes_05():
    staff = abjad.Staff(
        r"c'16 r8. d'16 r8. r8 r32 <e' g'>32 r16 r4 "
        r"\times 2/3 {r4 f'4 r4} r4 g'8 r8 a'4 ~ "
        r"a'16 r8. b'4 c''8 r8 "
        r"r4. d''8 \times 4/5 {r8 d''2}"
    )
    auxjad.mutate.sustain_notes(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'4
            ~
            d'8
            ~
            d'32
            <e' g'>16.
            ~
            <e' g'>4
            ~
            \times 2/3
            {
                <e' g'>4
                f'2
                ~
            }
            f'4
            g'4
            a'2
            b'4
            c''4
            ~
            c''4.
            d''8
            ~
            \times 4/5
            {
                d''8
                d''2
            }
        }
        """
    )


def test_sustain_notes_06():
    staff = abjad.Staff(r"c'16 r8. c'16 r8. c'4 c'16 r8.")
    with pytest.raises(TypeError):
        auxjad.mutate.sustain_notes(staff[:])


def test_sustain_notes_07():
    staff = abjad.Staff(
        r"""
        r4 c'16 r8. d'16 r4..
        R1
        r4 e'4 r2
        \time 5/8 r8 f'4 r4
        R1 * 5/8
        r8 g'8 a'8 r4
        """
    )
    auxjad.mutate.sustain_notes(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r4
            c'4
            d'2
            ~
            d'1
            ~
            d'4
            e'2.
            ~
            \time 5/8
            e'8
            f'2
            ~
            f'4.
            ~
            f'4
            f'8
            g'4
            a'4
        }
        """
    )


def test_sustain_notes_08():
    staff = abjad.Staff(
        r"""
        r4 c'16 r8. d'16 r4..
        R1
        r4 e'4 r2
        \time 5/8 r8 f'4 r4
        R1 * 5/8
        r8 g'8 a'8 r4
        """
    )
    auxjad.mutate.sustain_notes(staff, sustain_multimeasure_rests=False)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r4
            c'4
            d'2
            R1
            r4
            e'2.
            ~
            \time 5/8
            e'8
            f'2
            R1 * 5/8
            r8
            g'8
            a'4.
        }
        """
    )


def test_sustain_notes_09():
    staff = abjad.Staff(
        r"""
        r4 c'16 r8. d'16 r4..
        R1
        r4 e'4 r2
        \time 5/8 r8 f'4 r4
        R1 * 5/8
        r8 g'8 a'8 r4
        """
    )
    auxjad.mutate.sustain_notes(staff, rewrite_meter=False)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r4
            c'16
            ~
            c'8.
            d'16
            ~
            d'4..
            ~
            d'1
            ~
            d'4
            e'4
            ~
            e'2
            ~
            \time 5/8
            e'8
            f'4
            ~
            f'4
            ~
            f'2
            ~
            f'8
            f'8
            g'8
            ~
            a'8
            a'4
        }
        """
    )



import abjad

import auxjad


def test_selections_are_identical_01():
    container1 = abjad.Staff(r"c'4 d'4 e'4 f'4 <g' a'>2 r2")
    container2 = abjad.Staff(r"c'4 d'4 e'4 f'4 <g' a'>2 r2")
    selections = [container1[:], container2[:]]
    assert auxjad.get.selections_are_identical(selections)


def test_selections_are_identical_02():
    container1 = abjad.Staff(r"c'4 d'4 e'4 f'4 <g' a'>2 r2")
    container2 = abjad.Staff(r"\times 3/2 {c'4 d'4 e'4} f'4 <g' a'>2 r2")
    selections = [container1[:], container2[:]]
    assert not auxjad.get.selections_are_identical(selections)


def test_selections_are_identical_03():
    container1 = abjad.Staff(r"c'4\pp d'4 e'4-. f'4 <g' a'>2-> r2")
    container2 = abjad.Staff(r"c'4 d'4 e'4 f'4 <g' a'>2 r2")
    selections = [container1[:], container2[:]]
    assert not auxjad.get.selections_are_identical(selections)


def test_selections_are_identical_04():
    container1 = abjad.Staff(r"c'4\pp d'4 e'4-. f'4 <g' a'>2-> r2")
    container2 = abjad.Staff(r"c'4 d'4 e'4 f'4 <g' a'>2 r2")
    selections = [container1[:], container2[:]]
    assert auxjad.get.selections_are_identical(
        selections,
        include_indicators=False,
    )


def test_selections_are_identical_05():
    container1 = abjad.Staff(r"c'4\pp d'4 e'4-. f'4 <g' a'>2-> r2")
    container2 = abjad.Staff(r"c'4\pp d'4 e'4-. f'4 <g' a'>2-> r2")
    selections = [container1[:], container2[:]]
    assert auxjad.get.selections_are_identical(
        selections,
        include_indicators=True,
    )


def test_selections_are_identical_06():
    container1 = abjad.Staff(r"c'4 d'4 e'4 f'4 <g' a'>2 r2")
    container2 = abjad.Staff(r"c'4 \grace{d'4} e'4 f'4 <g' a'>2 r2")
    selections = [container1[:], container2[:]]
    assert not auxjad.get.selections_are_identical(selections)


def test_selections_are_identical_07():
    container1 = abjad.Staff(r"c'4 d'4 e'4 f'4 <g' a'>2 r2")
    container2 = abjad.Staff(r"c'4 \grace{c''4} d'4 e'4 f'4 <g' a'>2 r2")
    selections = [container1[:], container2[:]]
    assert not auxjad.get.selections_are_identical(selections)


def test_selections_are_identical_08():
    container1 = abjad.Staff(r"c'4 \grace{c''16} d'4 e'4 f'4 <g' a'>2 r2")
    container2 = abjad.Staff(r"c'4 \grace{c''16} d'4 e'4 f'4 <g' a'>2 r2")
    selections = [container1[:], container2[:]]
    assert auxjad.get.selections_are_identical(selections)


def test_selections_are_identical_09():
    container1 = abjad.Staff(r"c'4 \grace{c''4} d'4 e'4 f'4 <g' a'>2 r2")
    container2 = abjad.Staff(r"c'4 \grace{c''8} d'4 e'4 f'4 <g' a'>2 r2")
    selection1 = abjad.select(container1)  # x[:] doesn't include graces
    selection2 = abjad.select(container2)
    selections = [selection1, selection2]
    assert not auxjad.get.selections_are_identical(selections)


def test_selections_are_identical_10():
    container1 = abjad.Staff(r"c'4 \grace{c''4} d'4 e'4 f'4 <g' a'>2 r2")
    container2 = abjad.Staff(r"c'4 \grace{b4} d'4 e'4 f'4 <g' a'>2 r2")
    selection1 = abjad.select(container1)  # x[:] doesn't include graces
    selection2 = abjad.select(container2)
    selections = [selection1, selection2]
    assert not auxjad.get.selections_are_identical(selections)


def test_selections_are_identical_11():
    container1 = abjad.Container(r"c'4 d'4 e'4 f'4")
    container2 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    selections = [container1[:], container2[:]]
    assert auxjad.get.selections_are_identical(selections)


def test_selections_are_identical_12():
    container1 = abjad.Staff(r"c'4 d'4 e'4 f'4 <g' a'>2 r2")
    container2 = abjad.Staff(r"c'4 d'4 e'4 f'4 <g' a'>2 r2")
    selections = [container1[:], container2[:]]
    assert abjad.get.selections_are_identical(selections)


def test_selections_are_identical_13():
    container1 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    container2 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    abjad.piano_pedal(container1[:])
    abjad.piano_pedal(container2[:])
    selections = [container1[:], container2[:]]
    assert auxjad.get.selections_are_identical(selections)


def test_selections_are_identical_14():
    container1 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    container2 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    abjad.piano_pedal(container1[:])
    abjad.piano_pedal(container2[:-2])
    selections = [container1[:], container2[:]]
    assert not abjad.get.selections_are_identical(selections)



import abjad
import pytest

import auxjad


def test_sync_containers_01():
    container1 = abjad.Staff(r"\time 4/4 g'2.")
    container2 = abjad.Staff(r"\time 4/4 c'1")
    auxjad.mutate.sync_containers([container1, container2])
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            g'2.
            r4
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
        }
        """
    )


def test_sync_containers_02():
    container1 = abjad.Staff(r"\time 4/4 g'1 | f'4")
    container2 = abjad.Staff(r"\time 4/4 c'1")
    auxjad.mutate.sync_containers([container1, container2])
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            g'1
            \time 1/4
            f'4
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            \time 1/4
            R1 * 1/4
        }
        """
    )


def test_sync_containers_03():
    container1 = abjad.Staff(r"\time 4/4 g'1 | f'4")
    container2 = abjad.Staff(r"\time 4/4 c'1 | d'1")
    auxjad.mutate.sync_containers([container1, container2])
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            g'1
            f'4
            r2.
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            d'1
        }
        """
    )


def test_sync_containers_04():
    container1 = abjad.Staff(r"\time 4/4 g'1")
    container2 = abjad.Staff(r"\time 4/4 c'1 | d'1")
    auxjad.mutate.sync_containers([container1, container2])
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            g'1
            R1
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            d'1
        }
        """
    )


def test_sync_containers_05():
    container1 = abjad.Staff(r"\time 4/4 g'1")
    container2 = abjad.Staff(r"\time 4/4 c'1 | d'1")
    auxjad.mutate.sync_containers(
        [container1, container2],
        use_multimeasure_rests=False,
    )
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            g'1
            r1
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            d'1
        }
        """
    )


def test_sync_containers_06():
    container1 = abjad.Staff(r"\time 3/4 g'2.")
    container2 = abjad.Staff(r"\time 3/4 c'2. | d'2.")
    auxjad.mutate.sync_containers([container1, container2])
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            g'2.
            R1 * 3/4
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2.
            d'2.
        }
        """
    )


def test_sync_containers_07():
    container1 = abjad.Staff(r"\time 4/4 c'1 | g'4")
    container2 = abjad.Staff(r"\time 4/4 c'1 | g'2")
    container3 = abjad.Staff(r"\time 4/4 c'1 | g'2.")
    container4 = abjad.Staff(r"\time 4/4 c'1")
    containers = [container1, container2, container3, container4]
    auxjad.mutate.sync_containers(containers)
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2)
            == abjad.get.duration(container3)
            == abjad.get.duration(container4))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            \time 3/4
            g'4
            r2
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            \time 3/4
            g'2
            r4
        }
        """
    )
    assert abjad.lilypond(container3) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            \time 3/4
            g'2.
        }
        """
    )
    assert abjad.lilypond(container4) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            \time 3/4
            R1 * 3/4
        }
        """
    )


def test_sync_containers_08():
    container1 = abjad.Staff(r"\time 4/4 c'1 | g'4")
    container2 = abjad.Staff(r"\time 4/4 c'1 | g'2")
    container3 = abjad.Staff(r"\time 4/4 c'1 | g'2.")
    container4 = abjad.Staff(r"\time 4/4 c'1")
    containers = [container1,
                  container2,
                  container3,
                  container4,
                  ]
    auxjad.mutate.sync_containers(
        containers,
        use_multimeasure_rests=False,
    )
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2)
            == abjad.get.duration(container3)
            == abjad.get.duration(container4))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            \time 3/4
            g'4
            r2
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            \time 3/4
            g'2
            r4
        }
        """
    )
    assert abjad.lilypond(container3) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            \time 3/4
            g'2.
        }
        """
    )
    assert abjad.lilypond(container4) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            \time 3/4
            r2.
        }
        """
    )


def test_sync_containers_09():
    container1 = abjad.Staff(r"\time 4/4 c'1 | g'4")
    container2 = abjad.Staff(r"\time 4/4 c'1 | g'2")
    container3 = abjad.Staff(r"\time 4/4 c'1 | g'2.")
    container4 = abjad.Staff(r"\time 4/4 c'1")
    containers = [container1,
                  container2,
                  container3,
                  container4,
                  ]
    auxjad.mutate.sync_containers(
        containers,
        adjust_last_time_signature=False,
    )
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2)
            == abjad.get.duration(container3)
            == abjad.get.duration(container4))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            g'4
            r2
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            g'2
            r4
        }
        """
    )
    assert abjad.lilypond(container3) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            g'2.
        }
        """
    )
    assert abjad.lilypond(container4) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
            r2.
        }
        """
    )


def test_sync_containers_10():
    container1 = abjad.Staff(r"\time 4/4 c'4 d'4 e'4 f'4")
    container2 = abjad.Staff(r"\time 3/4 a2. \time 4/4 c'4")
    container3 = abjad.Staff(r"\time 5/4 g''1 ~ g''4")
    container4 = abjad.Staff(r"\time 6/8 c'2")
    containers = [container1, container2, container3, container4]
    auxjad.mutate.sync_containers(containers)
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2)
            == abjad.get.duration(container3)
            == abjad.get.duration(container4))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
            \time 1/4
            R1 * 1/4
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            a2.
            \time 2/4
            c'4
            r4
        }
        """
    )
    assert abjad.lilypond(container3) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/4
            g''1
            ~
            g''4
        }
        """
    )
    assert abjad.lilypond(container4) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 6/8
            c'2
            r4
            \time 2/4
            R1 * 1/2
        }
        """
    )


def test_sync_containers_11():
    container1 = abjad.Container(r"\time 4/4 g'1 | f'4")
    container2 = abjad.Container(r"\time 5/4 c'1 | \time 4/4 d'4")
    with pytest.raises(ValueError):
        auxjad.mutate.sync_containers([container1, container2])


def test_sync_containers_12():
    container1 = abjad.Staff(r"\time 4/4 g'2.")
    container2 = abjad.Staff(r"\time 4/4 c'1")
    auxjad.mutate.sync_containers([container1, container2])
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            g'2.
            r4
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'1
        }
        """
    )


def test_sync_containers_13():
    container1 = abjad.Staff(r"\time 3/4 g'2.")
    container2 = abjad.Staff(r"\time 3/4 c'2.")
    auxjad.mutate.sync_containers([container1, container2])
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            g'2.
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2.
        }
        """
    )


def test_sync_containers_14():
    container1 = abjad.Staff(r"\time 5/4 g'1~g'4 | R1 * 5/4")
    container2 = abjad.Staff(r"\time 5/4 c'2.")
    auxjad.mutate.sync_containers([container1, container2])
    assert (abjad.get.duration(container1)
            == abjad.get.duration(container2))
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/4
            g'1
            ~
            g'4
            R1 * 5/4
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/4
            c'2.
            r2
            R1 * 5/4
        }
        """
    )


def test_sync_containers_15():
    container1 = abjad.Staff(r"\time 7/4 a'1 ~ a'2.")
    container2 = abjad.Staff(r"\time 3/4 c'2.")
    auxjad.mutate.sync_containers([container1, container2])
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2.
            R1 * 3/4
            \time 1/4
            R1 * 1/4
        }
        """
    )


def test_sync_containers_16():
    staff1 = abjad.Staff(r"\time 3/8 c'4. | d'4")
    staff2 = abjad.Staff(r"\time 3/8 c'4. | d'8")
    staff3 = abjad.Staff(r"\time 3/8 c'4. | d'16")
    staff4 = abjad.Staff(r"\time 3/8 c'4.")
    score = abjad.Score([staff1,
                         staff2,
                         staff3,
                         staff4,
                         ])
    auxjad.mutate.sync_containers(score)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 3/8
                c'4.
                \time 1/4
                d'4
            }
            \new Staff
            {
                \time 3/8
                c'4.
                \time 1/4
                d'8
                r8
            }
            \new Staff
            {
                \time 3/8
                c'4.
                \time 1/4
                d'16
                r8.
            }
            \new Staff
            {
                \time 3/8
                c'4.
                \time 1/4
                R1 * 1/4
            }
        >>
        """
    )



import random

import abjad
import pytest

import auxjad


def test_Fader_01():
    random.seed(13987)
    container = abjad.Container(r"c'4 ~ c'16 d'8. e'8 f'4.")
    fader = auxjad.Fader(container)
    assert abjad.lilypond(fader) == abjad.String.normalize(
        r"""
        {
            %%% \time 4/4 %%%
            c'4
            ~
            c'16
            d'8.
            e'8
            f'4.
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            ~
            c'16
            d'8.
            e'8
            f'4.
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            ~
            c'16
            r8.
            e'8
            f'4.
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r2
            e'8
            f'4.
        }
        """
    )
    notes = fader.current_window
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r2
            e'8
            f'4.
        }
        """
    )


def test_Fader_02():
    random.seed(98752)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    fader = auxjad.Fader(container)
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            r4
            e'4
            f'4
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r2
            e'4
            f'4
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r2.
            f'4
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            R1
        }
        """
    )
    with pytest.raises(RuntimeError):
        notes = fader()  # noqa: F841


def test_Fader_03():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    fader = auxjad.Fader(container,
                         mode='in',
                         max_steps=2,
                         repetition_chance=0.7,
                         disable_rewrite_meter=True,
                         omit_time_signatures=True,
                         use_multimeasure_rests=False,
                         mask=[1, 0, 1, 1, 0],
                         boundary_depth=0,
                         maximum_dot_count=1,
                         rewrite_tuplets=False,
                         process_on_first_call=True,
                         include_empty_measures=False,
                         )
    assert fader.mode == 'in'
    assert fader.max_steps == 2
    assert fader.repetition_chance == 0.7
    assert fader.disable_rewrite_meter
    assert fader.omit_time_signatures
    assert not fader.use_multimeasure_rests
    assert fader.mask == [1, 0, 1, 1, 0]
    assert fader.boundary_depth == 0
    assert fader.maximum_dot_count == 1
    assert not fader.rewrite_tuplets
    assert fader.process_on_first_call
    assert not fader.include_empty_measures
    fader.mode = 'out'
    fader.max_steps = 1
    fader.repetition_chance = 0.23
    fader.disable_rewrite_meter = False
    fader.omit_time_signatures = False
    fader.use_multimeasure_rests = True
    fader.mask = [0, 1, 1, 0, 1]
    fader.boundary_depth = 1
    fader.maximum_dot_count = 2
    fader.rewrite_tuplets = True
    fader.process_on_first_call = False
    fader.include_empty_measures = True
    assert fader.mode == 'out'
    assert fader.max_steps == 1
    assert fader.repetition_chance == 0.23
    assert not fader.disable_rewrite_meter
    assert not fader.omit_time_signatures
    assert fader.use_multimeasure_rests
    assert fader.mask == [0, 1, 1, 0, 1]
    assert fader.boundary_depth == 1
    assert fader.maximum_dot_count == 2
    assert fader.rewrite_tuplets
    assert not fader.process_on_first_call
    assert fader.include_empty_measures


def test_Fader_04():
    random.seed(19962)
    container = abjad.Container(r"c'4. d'8 e'2")
    fader = auxjad.Fader(container)
    notes = fader.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4.
            d'8
            e'2
            r4.
            d'8
            e'2
            r2
            e'2
            R1
        }
        """
    )


def test_Fader_05():
    random.seed(98738)
    container = abjad.Container(r"c'4. d'8 e'2")
    fader = auxjad.Fader(container,
                         mode='in',
                         )
    notes = fader.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            R1
            r4.
            d'8
            r2
            c'4.
            d'8
            r2
            c'4.
            d'8
            e'2
        }
        """
    )


def test_Fader_06():
    random.seed(13241)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    fader = auxjad.Fader(container)
    notes = fader.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    notes = fader.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            r4
            f'4
        }
        """
    )
    notes = fader.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            r2
        }
        """
    )
    fader.mode = 'in'
    notes = fader.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            r4
            f'4
        }
        """
    )
    notes = fader.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    fader.mask = [0, 0, 1, 1]
    notes = fader.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r2
            e'4
            f'4
        }
        """
    )
    notes = fader.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r4
            d'4
            e'4
            f'4
        }
        """
    )


def test_Fader_07():
    random.seed(44126)
    container = abjad.Container(r"\times 2/3 {c'8 d'8 e'8} d'2.")
    fader = auxjad.Fader(container)
    notes = fader.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                \time 4/4
                c'8
                d'8
                e'8
            }
            d'2.
            \times 2/3
            {
                r8
                d'8
                e'8
            }
            d'2.
            \times 2/3
            {
                r8
                d'8
                r8
            }
            d'2.
            r4
            d'2.
            R1
        }
        """
    )


def test_Fader_08():
    random.seed(88111)
    container = abjad.Container(r"c'4. d'8 e'16 f'16 g'4.")
    fader = auxjad.Fader(container)
    notes = fader.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4.
            d'8
            e'16
            f'16
            g'4.
            c'4.
            r8
            e'16
            f'16
            g'4.
            c'4.
            r8
            e'16
            f'16
            r4.
        }
        """
    )


def test_Fader_09():
    random.seed(14812)
    container = abjad.Container(
        r"\time 3/8 c'4. \time 2/4 d'2 \time 3/8 e'4."
    )
    fader = auxjad.Fader(container)
    notes = fader.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/8
            c'4.
            \time 2/4
            d'2
            \time 3/8
            e'4.
            c'4.
            \time 2/4
            R1 * 1/2
            \time 3/8
            e'4.
            c'4.
            \time 2/4
            R1 * 1/2
            \time 3/8
            R1 * 3/8
        }
        """
    )


def test_Fader_10():
    random.seed(29862)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    fader = auxjad.Fader(container,
                         max_steps=3,
                         process_on_first_call=True,
                         )
    notes = fader.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'8
            r8
            f'8
            g'8
            a'8
            b'8
            c''8
            r4.
            f'8
            g'8
            a'8
            b'8
            c''8
            r4.
            f'8
            r8
            a'8
            b'8
            r8
        }
        """
    )


def test_Fader_11():
    random.seed(18711)
    container = abjad.Container(r"c'8 d'8 e'2.")
    fader = auxjad.Fader(container,
                         disable_rewrite_meter=True,
                         use_multimeasure_rests=False,
                         )
    notes = fader.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'8
            e'2.
            c'8
            r8
            e'2.
            r8
            r8
            e'2.
            r8
            r8
            r2.
        }
        """
    )


def test_Fader_12():
    random.seed(87123)
    container = abjad.Container(r"\time 2/4 c'4 d'4 \time 3/4 e'4 f'4 g'4")
    fader = auxjad.Fader(container,
                         omit_time_signatures=True,
                         )
    notes = fader.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'4
            e'4
            f'4
            g'4
            c'4
            d'4
            e'4
            f'4
            r4
            c'4
            d'4
            e'4
            r2
        }
        """
    )


def test_Fader_13():
    random.seed(47103)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    fader = auxjad.Fader(container,
                         process_on_first_call=True,
                         )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            r4
        }
        """
    )


def test_Fader_14():
    random.seed(19941)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    fader = auxjad.Fader(container,
                         mode='in',
                         mask=[0, 1, 1, 0]
                         )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r4
            d'4
            e'4
            r4
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            r4
        }
        """
    )
    fader.reset_mask()
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            R1
        }
        """
    )
    fader.mode = 'out'
    fader.reset_mask()
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_Fader_15():
    random.seed(71324)
    container = abjad.Container(
        r"\time 3/4 c'8->\f d'8\p ~ d'4 e'8..-- f'32-."
    )
    fader = auxjad.Fader(container)
    notes = fader.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'8
            \f
            - \accent
            d'4.
            \p
            e'8..
            - \tenuto
            f'32
            - \staccato
            c'8
            \f
            - \accent
            d'4.
            \p
            r8..
            f'32
            - \staccato
            c'8
            \f
            - \accent
            d'4.
            \p
            r4
            c'8
            \f
            - \accent
            r8
            r2
            R1 * 3/4
        }
        """
    )


def test_Fader_16():
    random.seed(91634)
    container = abjad.Container(r"c'4 ~ c'16 d'8. e'8 f'4.")
    fader = auxjad.Fader(container,
                         mode='in',
                         )
    assert abjad.lilypond(fader) == abjad.String.normalize(
        r"""
        {
            %%% \time 4/4 %%%
            c'4
            ~
            c'16
            d'8.
            e'8
            f'4.
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            R1
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r2
            r8
            f'4.
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            ~
            c'16
            r8.
            r8
            f'4.
        }
        """
    )
    notes = fader.current_window
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            ~
            c'16
            r8.
            r8
            f'4.
        }
        """
    )


def test_Fader_17():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    fader = auxjad.Fader(container)
    assert len(fader) == 4
    container = abjad.Container(r"c'4 ~ c'8 d'8 e'4 ~ e'8 f'8")
    fader = auxjad.Fader(container)
    assert len(fader) == 4
    container = abjad.Container(r"c'4 ~ c'16 r16 d'8 e'4 ~ e'8 f'16 r16")
    fader = auxjad.Fader(container)
    assert len(fader) == 4
    container = abjad.Container(r"<c' e' g'>2 <d' f'>2")
    fader = auxjad.Fader(container)
    assert len(fader) == 5
    container = abjad.Container(r"<c' e' g'>4 ~ <c' e' g'>16 r8. <d' f'>2")
    fader = auxjad.Fader(container)
    assert len(fader) == 5
    container = abjad.Container(r"<c' e' g'>4 d'4 <e' g' b'>4 r4")
    fader = auxjad.Fader(container)
    assert len(fader) == 7


def test_Fader_18():
    random.seed(66501)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    fader = auxjad.Fader(container)
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r4
            d'4
            e'4
            f'4
        }
        """
    )
    fader.contents = abjad.Container(r"c'16 d'16 e'16 f'16 g'2.")
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'16
            d'16
            e'16
            f'16
            g'2.
        }
        """
    )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'16
            d'16
            r16
            f'16
            g'2.
        }
        """
    )


def test_Fader_19():
    random.seed(48915)
    container = abjad.Container(r"c'4 d'8 e'8 f'4 ~ f'8. g'16")
    fader = auxjad.Fader(container)
    assert fader.mask == [1, 1, 1, 1, 1]
    fader = auxjad.Fader(container,
                         mode='in',
                         )
    assert fader.mask == [0, 0, 0, 0, 0]
    fader()
    assert fader.mask == [0, 0, 0, 0, 0]
    fader()
    assert fader.mask == [0, 1, 0, 0, 0]
    fader()
    assert fader.mask == [0, 1, 1, 0, 0]
    staff = abjad.Staff(fader.current_window)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r4
            d'8
            e'8
            r2
        }
        """
    )
    fader.mask = [1, 0, 1, 1, 0]
    assert fader.mask == [1, 0, 1, 1, 0]
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            r8
            e'8
            f'4..
            r16
        }
        """
    )
    fader.reset_mask()
    assert fader.mask == [0, 0, 0, 0, 0]
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            R1
        }
        """
    )


def test_Fader_20():
    container = abjad.Container(r"c'4. d'8 e'2")
    fader = auxjad.Fader(container)
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4.
            d'8
            e'2
        }
        """
    )
    fader = auxjad.Fader(container,
                         boundary_depth=1,
                         )
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            ~
            c'8
            d'8
            e'2
        }
        """
    )


def test_Fader_22():
    random.seed(92114)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    fader = auxjad.Fader(container)
    fader.random_mask()
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r8
            d'8
            r4
            g'8
            a'8
            r4
        }
        """
    )
    fader.random_mask()
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r8
            d'8
            r4
            g'8
            a'8
            b'8
            r8
        }
        """
    )


def test_Fader_23():
    random.seed(36017)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    fader = auxjad.Fader(container,
                         mask=[0, 0, 1, 1, 1, 1, 1, 1],
                         )
    fader.shuffle_mask()
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r8
            d'8
            e'8
            f'8
            g'8
            a'8
            b'8
            r8
        }
        """
    )
    fader.shuffle_mask()
    notes = fader()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'8
            e'8
            r8
            g'8
            r8
            b'8
            c''8
        }
        """
    )


def test_Fader_24():
    random.seed(83012)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    fader = auxjad.Fader(container)
    staff = abjad.Staff()
    for window in fader:
        staff.append(window)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
            \time 4/4
            c'4
            r4
            e'4
            f'4
            \time 4/4
            c'4
            r4
            e'4
            r4
            \time 4/4
            c'4
            r2.
            \time 4/4
            R1
        }
        """
    )
    auxjad.mutate.remove_repeated_time_signatures(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
            c'4
            r4
            e'4
            f'4
            c'4
            r4
            e'4
            r4
            c'4
            r2.
            R1
        }
        """
    )


def test_Fader_25():
    random.seed(19873)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    fader = auxjad.Fader(container, max_steps=3)
    staff = abjad.Staff()
    for window in fader:
        staff.append(window)
    auxjad.mutate.remove_repeated_time_signatures(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
            c'4
            r2.
            R1
        }
        """
    )


def test_Fader_26():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    fader = auxjad.Fader(container)
    assert isinstance(fader(), abjad.Selection)
    tuplet = abjad.Tuplet('3:2', r"c'2 d'2 e'2")
    fader = auxjad.Fader(tuplet)
    assert isinstance(fader(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    fader = auxjad.Fader(voice)
    assert isinstance(fader(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    fader = auxjad.Fader(staff)
    assert isinstance(fader(), abjad.Selection)
    score = abjad.Score([abjad.Staff(r"c'4 d'4 e'4 f'4")])
    fader = auxjad.Fader(score)
    assert isinstance(fader(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    staff = abjad.Staff([voice])
    fader = auxjad.Fader(staff)
    assert isinstance(fader(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    score = abjad.Score([staff])
    fader = auxjad.Fader(score)
    assert isinstance(fader(), abjad.Selection)

    voice1 = abjad.Voice(r"c'4 d'4 e'4 f'4")
    voice2 = abjad.Voice(r"g2 f2")
    staff = abjad.Staff([voice1, voice2], simultaneous=True)
    with pytest.raises(ValueError):
        fader = auxjad.Fader(staff)  # noqa: F841

    staff1 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    staff2 = abjad.Staff(r"g2 f2")
    score = abjad.Score([staff1, staff2])
    with pytest.raises(ValueError):
        fader = auxjad.Fader(score)  # noqa: F841


def test_Fader_27():
    random.seed(41888)
    container = abjad.Container(
        r"\times 2/3 {c'2(\p\< d'2 e'2\f} f'4\p\> g'2 a'4\pp)"
    )
    fader = auxjad.Fader(container)
    notes = fader.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                \time 4/4
                c'2
                \p
                \<
                (
                d'2
                e'2
                \f
            }
            f'4
            \p
            \>
            g'2
            a'4
            \pp
            )
            \times 2/3
            {
                c'2
                \p
                \<
                (
                d'2
                e'2
                \f
                )
            }
            r4
            g'2
            \p
            \>
            (
            a'4
            \pp
            )
            \times 2/3
            {
                r2
                d'2
                \p
                \<
                (
                e'2
                \f
                )
            }
            r4
            g'2
            \p
            \>
            (
            a'4
            \pp
            )
            \times 2/3
            {
                r2
                d'2
                \p
                \<
                r2
                \f
            }
            r4
            g'2
            \p
            \>
            (
            a'4
            \pp
            )
            R1
            r4
            g'2
            \p
            \>
            (
            a'4
            \pp
            )
        }
        """
    )


def test_Fader_28():
    random.seed(17613)
    container = abjad.Container(
        r"<c' e'>4 ~ <c' e'>16 d'8. <gs e'>8 <bf f' a'>8 ~ <bf f' a'>4"
    )
    fader = auxjad.Fader(container)
    staff = abjad.Staff(fader.output_all())
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            <c' e'>4
            ~
            <c' e'>16
            d'8.
            <gs e'>8
            <bf f' a'>4.
            <c' e'>4
            ~
            <c' e'>16
            d'8.
            gs8
            <bf f' a'>4.
            <c' e'>4
            ~
            <c' e'>16
            d'8.
            gs8
            <bf a'>4.
            c'4
            ~
            c'16
            d'8.
            gs8
            <bf a'>4.
            r4
            r16
            d'8.
            gs8
            <bf a'>4.
            r4
            r16
            d'8.
            gs8
            bf4.
            r2
            gs8
            bf4.
            r2
            r8
            bf4.
            R1
        }
        """
    )


def test_Fader_29():
    container = abjad.Container(r"c'2 <d' e' f' g'>2")
    fader = auxjad.Fader(container, mask=[1, 0, 1, 1, 0])
    staff = abjad.Staff(fader())
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'2
            <e' f'>2
        }
        """
    )


def test_Fader_30():
    random.seed(39761)
    container = abjad.Container([
        auxjad.ArtificialHarmonic(r"<c' f'>2"),
        abjad.Chord(r"<c' f'>2"),
    ])
    fader = auxjad.Fader(container, mode='out')
    staff = abjad.Staff(fader.output_all())
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            <
                c'
                \tweak style #'harmonic
                f'
            >2
            <c' f'>2
            <
                c'
                \tweak style #'harmonic
                f'
            >2
            f'2
            <
                c'
                \tweak style #'harmonic
                f'
            >2
            r2
            R1
        }
        """
    )


def test_Fader_31():
    random.seed(76132)
    container = abjad.Container(r"c'4 d'4 e'2")
    fader = auxjad.Fader(container,
                         mode='out',
                         process_on_first_call=False,
                         include_empty_measures=True,
                         )
    staff = abjad.Staff(fader.output_all())
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'2
            r4
            d'4
            e'2
            r2
            e'2
            R1
        }
        """
    )
    random.seed(76132)
    container = abjad.Container(r"c'4 d'4 e'2")
    fader = auxjad.Fader(container,
                         mode='out',
                         process_on_first_call=True,
                         include_empty_measures=True,
                         )
    staff = abjad.Staff(fader.output_all())
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r4
            d'4
            e'2
            r2
            e'2
            R1
        }
        """
    )
    random.seed(76132)
    container = abjad.Container(r"c'4 d'4 e'2")
    fader = auxjad.Fader(container,
                         mode='out',
                         process_on_first_call=False,
                         include_empty_measures=False,
                         )
    staff = abjad.Staff(fader.output_all())
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'2
            r4
            d'4
            e'2
            r2
            e'2
        }
        """
    )
    random.seed(76132)
    container = abjad.Container(r"c'4 d'4 e'2")
    fader = auxjad.Fader(container,
                         mode='out',
                         process_on_first_call=True,
                         include_empty_measures=False,
                         )
    staff = abjad.Staff(fader.output_all())
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r4
            d'4
            e'2
            r2
            e'2
        }
        """
    )


def test_Fader_32():
    random.seed(76132)
    container = abjad.Container(r"c'4 d'4 e'2")
    fader = auxjad.Fader(container,
                         mode='in',
                         process_on_first_call=False,
                         include_empty_measures=True,
                         )
    staff = abjad.Staff(fader.output_all())
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            R1
            c'4
            r2.
            c'4
            d'4
            r2
            c'4
            d'4
            e'2
        }
        """
    )
    random.seed(76132)
    container = abjad.Container(r"c'4 d'4 e'2")
    fader = auxjad.Fader(container,
                         mode='in',
                         process_on_first_call=True,
                         include_empty_measures=True,
                         )
    staff = abjad.Staff(fader.output_all())
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            r2.
            c'4
            d'4
            r2
            c'4
            d'4
            e'2
        }
        """
    )
    random.seed(76132)
    container = abjad.Container(r"c'4 d'4 e'2")
    fader = auxjad.Fader(container,
                         mode='in',
                         process_on_first_call=False,
                         include_empty_measures=False,
                         )
    staff = abjad.Staff(fader.output_all())
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            r2.
            c'4
            d'4
            r2
            c'4
            d'4
            e'2
        }
        """
    )
    random.seed(76132)
    container = abjad.Container(r"c'4 d'4 e'2")
    fader = auxjad.Fader(container,
                         mode='in',
                         process_on_first_call=True,
                         include_empty_measures=False,
                         )
    staff = abjad.Staff(fader.output_all())
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            r2.
            c'4
            d'4
            r2
            c'4
            d'4
            e'2
        }
        """
    )


def test_Fader_33():
    random.seed(85909)
    container = abjad.Container(r"c'4. d'8 e'4.. f'16")
    fader = auxjad.Fader(container,
                         repetition_chance=0.5,
                         )
    notes = fader.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4.
            d'8
            e'4..
            f'16
            c'4.
            d'8
            e'4..
            r16
            c'4.
            d'8
            e'4..
            r16
            c'4.
            d'8
            r2
            c'4.
            d'8
            r2
        }
        """
    )


def test_Fader_34():
    random.seed(53234)
    container = abjad.Container(r"\time 4/4 c'2( d'2 \time 3/4 e'2.)")
    fader = auxjad.Fader(container, mode='in')
    notes = fader.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            R1
            \time 3/4
            R1 * 3/4
            \time 4/4
            c'2
            r2
            )
            \time 3/4
            R1 * 3/4
            \time 4/4
            c'2
            (
            d'2
            )
            \time 3/4
            R1 * 3/4
            \time 4/4
            c'2
            (
            d'2
            \time 3/4
            e'2.
            )
        }
        """
    )



import abjad

import auxjad


def test_reposition_dynamics_01():
    staff = abjad.Staff(r"c'1\p d'2 r2\f r1 e'1")
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            d'2
            r2
            r1
            e'1
            \f
        }
        """
    )


def test_reposition_dynamics_02():
    staff = abjad.Staff(r"c'1\p d'1 e'1\f e'1")
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            d'1
            e'1
            \f
            e'1
        }
        """
    )


def test_reposition_dynamics_03():
    staff = abjad.Staff(r"c'1\p d'2 r2\f r1\mf e'1\pp")
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            d'2
            r2
            r1
            e'1
            \pp
        }
        """
    )


def test_reposition_dynamics_04():
    staff = abjad.Staff(r"c'1\p d'1 r1\f e'1\p")
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            d'1
            r1
            e'1
        }
        """
    )


def test_reposition_dynamics_05():
    staff = abjad.Staff(r"c'1\p d'1 e'1\p f'1")
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            d'1
            e'1
            f'1
        }
        """
    )
    staff = abjad.Staff(r"c'1\p d'1 e'1\p f'1")
    auxjad.mutate.reposition_dynamics(staff[:], remove_repeated_dynamics=False)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            d'1
            e'1
            \p
            f'1
        }
        """
    )


def test_reposition_dynamics_06():
    staff = abjad.Staff(r"c'1\p d'1 r1\f e'1\p")
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            d'1
            r1
            e'1
        }
        """
    )
    staff = abjad.Staff(r"c'1\p d'1 r1\f e'1\p")
    auxjad.mutate.reposition_dynamics(staff[:], remove_repeated_dynamics=False)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            d'1
            r1
            e'1
            \p
        }
        """
    )


def test_reposition_dynamics_07():
    staff = abjad.Staff(r"c'1\p\< d'2 r2 r1\f e'1")
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            \<
            d'2
            r2
            \!
            r1
            e'1
            \f
        }
        """
    )


def test_reposition_dynamics_08():
    staff = abjad.Staff(r"c'1\p\< d'2 r2 r1\f e'1")
    auxjad.mutate.reposition_dynamics(
        staff[:],
        allow_hairpins_under_rests=True,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            \<
            d'2
            r2
            r1
            e'1
            \f
        }
        """
    )


def test_reposition_dynamics_09():
    staff = abjad.Staff(r"c'1\p\> d'1\f\> e'1\p")
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            d'1
            \f
            \>
            e'1
            \p
        }
        """
    )
    staff = abjad.Staff(r"c'1\p\> d'1\f\> e'1\p")
    auxjad.mutate.reposition_dynamics(staff[:], check_hairpin_trends=False)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            \>
            d'1
            \f
            \>
            e'1
            \p
        }
        """
    )


def test_reposition_dynamics_10():
    staff = abjad.Staff(r"c'1\p\> d'1\! e'1\f\> f'1\p")
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            \>
            d'1
            \!
            e'1
            \f
            \>
            f'1
            \p
        }
        """
    )


def test_reposition_dynamics_11():
    staff = abjad.Staff(r"c'1 d'1 e'1 r1\mf r1\ff f'1 r1 g'1")
    abjad.attach(abjad.Dynamic('niente', hide=True), staff[0])
    abjad.attach(abjad.Dynamic('niente', hide=True), staff[7])
    abjad.attach(abjad.StartHairpin('o<'), staff[0])
    abjad.attach(abjad.StartHairpin('>o'), staff[4])
    abjad.attach(abjad.StopHairpin(), staff[7])
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            - \tweak circled-tip ##t
            \<
            d'1
            e'1
            r1
            \mf
            r1
            f'1
            \ff
            - \tweak circled-tip ##t
            \>
            r1
            \!
            g'1
        }
        """
    )


def test_reposition_dynamics_12():
    staff = abjad.Staff(r"c'1\p d'1\f\> e'1\ff\< r1\fff f'1\p\> g'1\ppp")
    abjad.attach(abjad.StartHairpin('--'), staff[0])
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            - \tweak stencil #constante-hairpin
            \<
            d'1
            \f
            e'1
            \ff
            \<
            r1
            \fff
            f'1
            \p
            \>
            g'1
            \ppp
        }
        """
    )


def test_reposition_dynamics_13():
    staff = abjad.Staff(r"c'1\p R1\f d'1")
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            R1
            d'1
            \f
        }
        """
    )


def test_reposition_dynamics_14():
    staff = abjad.Staff(r"c'1\p\< d'2 r2\f r1 e'1")
    auxjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            \<
            d'2
            r2
            \f
            r1
            e'1
        }
        """
    )
    staff = abjad.Staff(r"c'1\p\< d'2 r2\f r1 e'1")
    auxjad.mutate.reposition_dynamics(
        staff[:],
        allow_hairpin_to_rest_with_dynamic=False,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            \<
            d'2
            r2
            \!
            r1
            e'1
            \f
        }
        """
    )


def test_reposition_dynamics_15():
    staff = abjad.Staff(r"c'1\p d'2 r2\f r1 e'1")
    abjad.mutate.reposition_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            d'2
            r2
            r1
            e'1
            \f
        }
        """
    )



import abjad

import auxjad


def test_Score_add_double_bar_lines_before_time_signatures_01():
    staff = abjad.Staff(
        r"\time 3/4 c'2. \time 4/4 d'1 e'1 \time 6/4 f'2. g'2."
    )
    score = auxjad.Score([staff])
    score.add_double_bar_lines_before_time_signatures()
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 3/4
                c'2.
                \bar "||"
                \time 4/4
                d'1
                e'1
                \bar "||"
                \time 6/4
                f'2.
                g'2.
            }
        >>
        """
    )


def test_Score_add_double_bar_lines_before_time_signatures_02():
    staff_1 = abjad.Staff(
        r"\time 3/4 c''2. \time 4/4 d''1 e''1 \time 6/4 f''2. g''2."
    )
    staff_2 = abjad.Staff(
        r"\time 3/4 c'2. \time 4/4 d'1 e'1 \time 6/4 f'2. g'2."
    )
    score = auxjad.Score([staff_1, staff_2])
    score.add_double_bar_lines_before_time_signatures()
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 3/4
                c''2.
                \bar "||"
                \time 4/4
                d''1
                e''1
                \bar "||"
                \time 6/4
                f''2.
                g''2.
            }
            \new Staff
            {
                \time 3/4
                c'2.
                \bar "||"
                \time 4/4
                d'1
                e'1
                \bar "||"
                \time 6/4
                f'2.
                g'2.
            }
        >>
        """
    )
    assert abjad.lilypond(staff_1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c''2.
            \bar "||"
            \time 4/4
            d''1
            e''1
            \bar "||"
            \time 6/4
            f''2.
            g''2.
        }
        """
    )
    assert abjad.lilypond(staff_2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2.
            \bar "||"
            \time 4/4
            d'1
            e'1
            \bar "||"
            \time 6/4
            f'2.
            g'2.
        }
        """
    )


def test_Score_add_double_bar_lines_before_time_signatures_03():
    voice_1 = abjad.Voice(
        r"\time 3/4 c''2. \time 4/4 d''1 e''1 \time 6/4 f''2. g''2."
    )
    voice_2 = abjad.Voice(
        r"\time 3/4 c'2. \time 4/4 d'1 e'1 \time 6/4 f'2. g'2."
    )
    staff = abjad.Staff([voice_1, voice_2], simultaneous=True)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
    score = auxjad.Score([staff])
    score.add_double_bar_lines_before_time_signatures()
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            <<
                \new Voice
                {
                    \voiceOne
                    \time 3/4
                    c''2.
                    \time 4/4
                    d''1
                    e''1
                    \time 6/4
                    f''2.
                    g''2.
                }
                \new Voice
                {
                    \voiceTwo
                    \time 3/4
                    c'2.
                    \bar "||"
                    \time 4/4
                    d'1
                    e'1
                    \bar "||"
                    \time 6/4
                    f'2.
                    g'2.
                }
            >>
        >>
        """
    )


def test_Score_add_double_bar_lines_before_time_signatures_04():
    voice_1 = abjad.Voice(
        r"\time 3/4 c''2. \time 4/4 d''1 e''1 \time 6/4 f''2. g''2."
    )
    voice_2 = abjad.Voice(
        r"\time 3/4 c'2. \time 4/4 d'1 e'1 \time 6/4 f'2. g'2."
    )
    staff = abjad.Staff([voice_1, voice_2], simultaneous=True)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
    score = auxjad.Score([staff])
    score.add_double_bar_lines_before_time_signatures(to_each_voice=True)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            <<
                \new Voice
                {
                    \voiceOne
                    \time 3/4
                    c''2.
                    \bar "||"
                    \time 4/4
                    d''1
                    e''1
                    \bar "||"
                    \time 6/4
                    f''2.
                    g''2.
                }
                \new Voice
                {
                    \voiceTwo
                    \time 3/4
                    c'2.
                    \bar "||"
                    \time 4/4
                    d'1
                    e'1
                    \bar "||"
                    \time 6/4
                    f'2.
                    g'2.
                }
            >>
        >>
        """
    )


def test_Score_add_double_bar_lines_before_time_signatures_05():
    voice_1 = abjad.Voice(
        r"\time 3/4 c''2. \time 4/4 d''1 e''1 \time 6/4 f''2. g''2."
    )
    voice_2 = abjad.Voice(
        r"\time 3/4 c'2. \time 4/4 d'1 e'1 \time 6/4 f'2. g'2."
    )
    staff_1 = abjad.Staff([voice_1, voice_2], simultaneous=True)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
    staff_2 = abjad.Staff(
        r"\time 3/4 c'2. \time 4/4 d'1 e'1 \time 6/4 f'2. g'2."
    )
    score = auxjad.Score([staff_1, staff_2])
    score.add_double_bar_lines_before_time_signatures()
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            <<
                \new Voice
                {
                    \voiceOne
                    \time 3/4
                    c''2.
                    \time 4/4
                    d''1
                    e''1
                    \time 6/4
                    f''2.
                    g''2.
                }
                \new Voice
                {
                    \voiceTwo
                    \time 3/4
                    c'2.
                    \bar "||"
                    \time 4/4
                    d'1
                    e'1
                    \bar "||"
                    \time 6/4
                    f'2.
                    g'2.
                }
            >>
            \new Staff
            {
                \time 3/4
                c'2.
                \bar "||"
                \time 4/4
                d'1
                e'1
                \bar "||"
                \time 6/4
                f'2.
                g'2.
            }
        >>
        """
    )


def test_Score_add_double_bar_lines_before_time_signatures_06():
    voice_1 = abjad.Voice(
        r"\time 3/4 c''2. \time 4/4 d''1 e''1 \time 6/4 f''2. g''2."
    )
    voice_2 = abjad.Voice(
        r"\time 3/4 c'2. \time 4/4 d'1 e'1 \time 6/4 f'2. g'2."
    )
    staff_1 = abjad.Staff([voice_1, voice_2], simultaneous=True)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
    staff_2 = abjad.Staff(
        r"\time 3/4 c'2. \time 4/4 d'1 e'1 \time 6/4 f'2. g'2."
    )
    score = auxjad.Score([staff_1, staff_2])
    score.add_double_bar_lines_before_time_signatures(to_each_voice=True)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            <<
                \new Voice
                {
                    \voiceOne
                    \time 3/4
                    c''2.
                    \bar "||"
                    \time 4/4
                    d''1
                    e''1
                    \bar "||"
                    \time 6/4
                    f''2.
                    g''2.
                }
                \new Voice
                {
                    \voiceTwo
                    \time 3/4
                    c'2.
                    \bar "||"
                    \time 4/4
                    d'1
                    e'1
                    \bar "||"
                    \time 6/4
                    f'2.
                    g'2.
                }
            >>
            \new Staff
            {
                \time 3/4
                c'2.
                \bar "||"
                \time 4/4
                d'1
                e'1
                \bar "||"
                \time 6/4
                f'2.
                g'2.
            }
        >>
        """
    )


def test_Score_add_double_bar_lines_before_time_signatures_07():
    staff = abjad.Staff(
        r"\time 3/4 R1 * 3/4 \time 4/4 R1 * 2 \time 6/4 R1 * 6/4 \time 4/4 R1"
    )
    score = auxjad.Score([staff])
    score.add_double_bar_lines_before_time_signatures()
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 3/4
                R1 * 3/4
                \bar "||"
                \time 4/4
                R1 * 2
                \bar "||"
                \time 6/4
                R1 * 3/2
                \bar "||"
                \time 4/4
                R1
            }
        >>
        """
    )


def test_Score_add_double_bar_lines_before_time_signatures_08():
    staff = abjad.Staff(
        r"R1 "
        r"\time 3/4 c'2. "
        r"\time 4/4 d'1 "
        r"e'1 "
        r"\time 6/4 f'2. g'2. "
        r"\time 2/4 a'2"
    )
    abjad.attach(abjad.BarLine('.|:'), staff[0])
    abjad.attach(abjad.BarLine(':|.'), staff[1])
    abjad.attach(abjad.BarLine('|'), staff[3])
    abjad.attach(abjad.BarLine('!'), staff[5])
    score = auxjad.Score([staff])
    score.add_double_bar_lines_before_time_signatures()
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                R1
                \bar ".|:"
                \time 3/4
                c'2.
                \bar ":|."
                \time 4/4
                d'1
                e'1
                \bar "||"
                \time 6/4
                f'2.
                g'2.
                \bar "!"
                \time 2/4
                a'2
            }
        >>
        """
    )


def test_Score_add_double_bar_lines_before_time_signatures_09():
    assert (auxjad.Score.add_double_bar_lines_before_time_signatures
            is abjad.Score.add_double_bar_lines_before_time_signatures)



import random

import abjad

import auxjad


def test_example_of_usage_02():
    random.seed(18762)
    pitch_selector = auxjad.TenneySelector(["c'", "fs'", "bf'",
                                            "e''", "a''", "d'''"])
    duration_selector = auxjad.TenneySelector([(1, 16),
                                               (2, 16),
                                               (3, 16),
                                               (4, 16),
                                               (5, 16),
                                               (6, 16),
                                               ])
    pitches = []
    durations = []
    for _ in range(12):
        pitches.append(pitch_selector())
        durations.append(duration_selector())
    notes = abjad.LeafMaker()(pitches, durations)
    container = abjad.Container(notes)
    looper = auxjad.LeafLooper(container, window_size=4)
    staff = abjad.Staff(looper.output_n(7))
    looper.window_size = 2
    staff.append(looper.output_n(4))
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 11/16
            a''16
            d'''8
            ~
            d'''8.
            c'16
            ~
            c'8.
            e''16
            \time 7/8
            d'''4
            ~
            d'''16
            c'16
            ~
            c'8.
            e''16
            bf'4
            \time 3/4
            c'4
            e''16
            bf'8.
            ~
            bf'16
            fs'8.
            \time 5/8
            e''16
            bf'16
            ~
            bf'8.
            fs'16
            ~
            fs'8
            a''8
            \time 7/8
            bf'4
            fs'8
            ~
            fs'16
            a''8
            d'''16
            ~
            d'''4
            \time 11/16
            fs'8.
            a''8
            d'''4
            ~
            d'''16
            c'16
            \time 3/4
            a''8
            d'''8
            ~
            d'''8.
            c'16
            d'''4
            \time 3/8
            d'''4
            ~
            d'''16
            c'16
            \time 5/16
            c'16
            d'''4
            \time 5/8
            d'''4
            a''4.
            \time 11/16
            a''4.
            e''16
            ~
            e''4
        }
        """
    )



import abjad

import auxjad


def test_Score_add_final_bar_line_01():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    score = auxjad.Score([staff])
    score.add_final_bar_line()
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c'4
                d'4
                e'4
                f'4
                \bar "|."
            }
        >>
        """
    )


def test_Score_add_final_bar_line_02():
    staff_1 = abjad.Staff(r"c''1 d''1 e''1 f''1")
    staff_2 = abjad.Staff(r"c'1 d'1 e'1 f'1")
    score = auxjad.Score([staff_1, staff_2])
    score.add_final_bar_line()
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c''1
                d''1
                e''1
                f''1
                \bar "|."
            }
            \new Staff
            {
                c'1
                d'1
                e'1
                f'1
                \bar "|."
            }
        >>
        """
    )
    assert abjad.lilypond(staff_1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c''1
            d''1
            e''1
            f''1
            \bar "|."
        }
        """
    )
    assert abjad.lilypond(staff_2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            d'1
            e'1
            f'1
            \bar "|."
        }
        """
    )


def test_Score_add_final_bar_line_03():
    voice_1 = abjad.Voice(r"c''1 d''1 e''1 f''1")
    voice_2 = abjad.Voice(r"c'2 d'2 e'2 f'2 g'2 a'2 b'2 c''2")
    staff = abjad.Staff([voice_1, voice_2], simultaneous=True)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
    score = auxjad.Score([staff])
    score.add_final_bar_line()
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            <<
                \new Voice
                {
                    \voiceOne
                    c''1
                    d''1
                    e''1
                    f''1
                }
                \new Voice
                {
                    \voiceTwo
                    c'2
                    d'2
                    e'2
                    f'2
                    g'2
                    a'2
                    b'2
                    c''2
                    \bar "|."
                }
            >>
        >>
        """
    )


def test_Score_add_final_bar_line_04():
    voice_1 = abjad.Voice(r"c''1 d''1 e''1 f''1")
    voice_2 = abjad.Voice(r"c'2 d'2 e'2 f'2 g'2 a'2 b'2 c''2")
    staff = abjad.Staff([voice_1, voice_2], simultaneous=True)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
    score = auxjad.Score([staff])
    score.add_final_bar_line(to_each_voice=True)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            <<
                \new Voice
                {
                    \voiceOne
                    c''1
                    d''1
                    e''1
                    f''1
                    \bar "|."
                }
                \new Voice
                {
                    \voiceTwo
                    c'2
                    d'2
                    e'2
                    f'2
                    g'2
                    a'2
                    b'2
                    c''2
                    \bar "|."
                }
            >>
        >>
        """
    )


def test_Score_add_final_bar_line_05():
    voice_1 = abjad.Voice(r"c''1 d''1 e''1 f''1")
    voice_2 = abjad.Voice(r"c'2 d'2 e'2 f'2 g'2 a'2 b'2 c''2")
    staff_1 = abjad.Staff([voice_1, voice_2], simultaneous=True)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
    staff_2 = abjad.Staff(r"c'1 d'1 e'1 f'1")
    score = auxjad.Score([staff_1, staff_2])
    score.add_final_bar_line()
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            <<
                \new Voice
                {
                    \voiceOne
                    c''1
                    d''1
                    e''1
                    f''1
                }
                \new Voice
                {
                    \voiceTwo
                    c'2
                    d'2
                    e'2
                    f'2
                    g'2
                    a'2
                    b'2
                    c''2
                    \bar "|."
                }
            >>
            \new Staff
            {
                c'1
                d'1
                e'1
                f'1
                \bar "|."
            }
        >>
        """
    )


def test_Score_add_final_bar_line_06():
    voice_1 = abjad.Voice(r"c''1 d''1 e''1 f''1")
    voice_2 = abjad.Voice(r"c'2 d'2 e'2 f'2 g'2 a'2 b'2 c''2")
    staff_1 = abjad.Staff([voice_1, voice_2], simultaneous=True)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
    staff_2 = abjad.Staff(r"c'1 d'1 e'1 f'1")
    score = auxjad.Score([staff_1, staff_2])
    score.add_final_bar_line(to_each_voice=True)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            <<
                \new Voice
                {
                    \voiceOne
                    c''1
                    d''1
                    e''1
                    f''1
                    \bar "|."
                }
                \new Voice
                {
                    \voiceTwo
                    c'2
                    d'2
                    e'2
                    f'2
                    g'2
                    a'2
                    b'2
                    c''2
                    \bar "|."
                }
            >>
            \new Staff
            {
                c'1
                d'1
                e'1
                f'1
                \bar "|."
            }
        >>
        """
    )


def test_Score_add_final_bar_line_07():
    voice_1 = abjad.Voice(r"c''1 d''1 e''1 f''1")
    voice_2 = abjad.Voice(r"c'1 d'1 e'1 f'1")
    staff = abjad.Staff([voice_1, voice_2], simultaneous=True)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
    score = auxjad.Score([staff])
    score.add_final_bar_line()
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            <<
                \new Voice
                {
                    \voiceOne
                    c''1
                    d''1
                    e''1
                    f''1
                }
                \new Voice
                {
                    \voiceTwo
                    c'1
                    d'1
                    e'1
                    f'1
                    \bar "|."
                }
            >>
        >>
        """
    )


def test_Score_add_final_bar_line_08():
    staff_1 = abjad.Staff(r"c''1 d''1 e''1 f''1")
    staff_2 = abjad.Staff(r"c'1 d'1 e'1 f'1")
    score = auxjad.Score([staff_1, staff_2])
    score.add_final_bar_line(abjad.BarLine(':|.'))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c''1
                d''1
                e''1
                f''1
                \bar ":|."
            }
            \new Staff
            {
                c'1
                d'1
                e'1
                f'1
                \bar ":|."
            }
        >>
        """
    )


def test_Score_add_final_bar_line_09():
    staff_1 = abjad.Staff(r"c''1 d''1 e''1 f''1")
    staff_2 = abjad.Staff(r"c'1 d'1 e'1 f'1")
    score = auxjad.Score([staff_1, staff_2])
    score.add_final_bar_line("||")
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c''1
                d''1
                e''1
                f''1
                \bar "||"
            }
            \new Staff
            {
                c'1
                d'1
                e'1
                f'1
                \bar "||"
            }
        >>
        """
    )


def test_Score_add_final_bar_line_10():
    voice_1 = abjad.Voice(r"c''1 d''1 e''1 f''1")
    voice_2 = abjad.Voice(r"c'1 d'1 e'1")
    staff = abjad.Staff([voice_1, voice_2], simultaneous=True)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
    abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
    score = auxjad.Score([staff])
    score.add_final_bar_line(to_each_voice=True)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            <<
                \new Voice
                {
                    \voiceOne
                    c''1
                    d''1
                    e''1
                    f''1
                    \bar "|."
                }
                \new Voice
                {
                    \voiceTwo
                    c'1
                    d'1
                    e'1
                    \bar "|."
                }
            >>
        >>
        """
    )


def test_Score_add_final_bar_line_11():
    assert auxjad.Score.add_final_bar_line is abjad.Score.add_final_bar_line



import abjad
import pytest

import auxjad


def test_fill_with_rests_01():
    container1 = abjad.Container(r"c'4 d'4 e'4 f'4")
    container2 = abjad.Container(r"c'4 d'4 e'4")
    container3 = abjad.Container(r"c'4 d'4 e'4 f'4 | c'4")
    container4 = abjad.Container(r"c'4 d'4 e'4 f'4 | c'4 d'4 e'4 f'4")
    auxjad.mutate.fill_with_rests(container1)
    auxjad.mutate.fill_with_rests(container2)
    auxjad.mutate.fill_with_rests(container3)
    auxjad.mutate.fill_with_rests(container4)
    assert abjad.lilypond(container1) == abjad.String.normalize(
        r"""
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    assert abjad.lilypond(container2) == abjad.String.normalize(
        r"""
        {
            c'4
            d'4
            e'4
            r4
        }
        """
    )
    assert abjad.lilypond(container3) == abjad.String.normalize(
        r"""
        {
            c'4
            d'4
            e'4
            f'4
            c'4
            r2.
        }
        """
    )
    assert abjad.lilypond(container4) == abjad.String.normalize(
        r"""
        {
            c'4
            d'4
            e'4
            f'4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_fill_with_rests_02():
    staff1 = abjad.Staff(r"\time 4/4 c'4 d'4 e'4 f'4 g'")
    staff2 = abjad.Staff(r"\time 3/4 a2. \time 2/4 c'4")
    staff3 = abjad.Staff(r"\time 5/4 g1 ~ g4 \time 4/4 af'2")
    auxjad.mutate.fill_with_rests(staff1)
    auxjad.mutate.fill_with_rests(staff2)
    auxjad.mutate.fill_with_rests(staff3)
    assert abjad.lilypond(staff1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
            g'4
            r2.
        }
        """
    )
    assert abjad.lilypond(staff2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            a2.
            \time 2/4
            c'4
            r4
        }
        """
    )
    assert abjad.lilypond(staff3) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/4
            g1
            ~
            g4
            \time 4/4
            af'2
            r2
        }
        """
    )


def test_fill_with_rests_03():
    container = abjad.Container(r"\time 4/4 c'4 d'4 e'4 f'4 g'4")
    auxjad.mutate.fill_with_rests(container)
    assert abjad.lilypond(container) == abjad.String.normalize(
        r"""
        {
            %%% \time 4/4 %%%
            c'4
            d'4
            e'4
            f'4
            g'4
            r2.
        }
        """
    )
    staff = abjad.Staff([container])
    assert abjad.lilypond(container) == abjad.String.normalize(
        r"""
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
            g'4
            r2.
        }
        """
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            {
                \time 4/4
                c'4
                d'4
                e'4
                f'4
                g'4
                r2.
            }
        }
        """
    )


def test_fill_with_rests_04():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4 g'4")
    time_signature = abjad.TimeSignature((3, 4), partial=(1, 4))
    abjad.attach(time_signature, staff[0])
    auxjad.mutate.fill_with_rests(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \partial 4
            \time 3/4
            c'4
            d'4
            e'4
            f'4
            g'4
            r2
        }
        """
    )


def test_fill_with_rests_05():
    container = abjad.Container(r"\time 5/4 g''1 \time 4/4 f'1")
    with pytest.raises(ValueError):
        auxjad.mutate.fill_with_rests(container)


def test_fill_with_rests_06():
    staff = abjad.Staff(r"\time 4/4 c'4 d'4 e'4 f'4 g'4 ~ g'16")
    auxjad.mutate.fill_with_rests(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
            g'4
            ~
            g'16
            r8.
            r2
        }
        """
    )


def test_fill_with_rests_07():
    staff = abjad.Staff(r"\time 4/4 c'8 d'4 e'4")
    auxjad.mutate.fill_with_rests(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'4
            e'8
            ~
            e'8
            r4.
        }
        """
    )
    staff = abjad.Staff(r"\time 4/4 c'8 d'4 e'4")
    auxjad.mutate.fill_with_rests(staff, disable_rewrite_meter=True)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'4
            e'4
            r4.
        }
        """
    )



import abjad
import pytest

import auxjad


def test_Repeater_01():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    repeater = auxjad.Repeater(container)
    assert abjad.lilypond(repeater) == abjad.String.normalize(
        r"""
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    notes = repeater()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    notes = repeater.current_window
    with pytest.raises(AttributeError):
        repeater.current_window = abjad.Container(r"c''2 e''2")
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_Repeater_02():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    repeater = auxjad.Repeater(container)
    notes = repeater(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'4
            e'4
            f'4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_Repeater_03():
    container = abjad.Container(r"\time 3/4 c'2. \time 2/4 r2 g'2")
    repeater = auxjad.Repeater(container)
    notes = repeater(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'2.
            \time 2/4
            r2
            g'2
            \time 3/4
            c'2.
            \time 2/4
            r2
            g'2
            \time 3/4
            c'2.
            \time 2/4
            r2
            g'2
        }
        """
    )


def test_Repeater_04():
    container = abjad.Container(r"c'4 d'4 e'4")
    repeater = auxjad.Repeater(container)
    notes = repeater(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            c'4
            d'4
            e'4
            c'4
            d'4
            e'4
        }
        """
    )


def test_Repeater_05():
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4 f'2")
    repeater = auxjad.Repeater(container)
    notes = repeater(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            \time 2/4
            f'2
            \time 3/4
            c'4
            d'4
            e'4
            \time 2/4
            f'2
        }
        """
    )


def test_Repeater_06():
    container = abjad.Container(r"c'4 d'4 e'4")
    repeater = auxjad.Repeater(container,
                               omit_time_signatures=True,
                               )
    notes = repeater(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'4
            e'4
            c'4
            d'4
            e'4
            c'4
            d'4
            e'4
        }
        """
    )


def test_Repeater_07():
    container = abjad.Container(r"\time 5/4 c'2. d'4 e'4")
    repeater = auxjad.Repeater(container,
                               force_identical_time_signatures=True,
                               )
    notes = repeater(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/4
            c'2.
            d'4
            e'4
            \time 5/4
            c'2.
            d'4
            e'4
            \time 5/4
            c'2.
            d'4
            e'4
        }
        """
    )


def test_Repeater_08():
    container = abjad.Container(r"\time 5/4 g''1 \time 4/4 f'1")
    with pytest.raises(ValueError):
        repeater = auxjad.Repeater(container)  # noqa: F841


def test_Repeater_09():
    container = abjad.Container(r"\clef bass f4\pp( e4) d4(")
    repeater = auxjad.Repeater(container)
    notes = repeater(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            \clef "bass"
            f4
            \pp
            (
            e4
            )
            d4
            (
            f4
            e4
            )
            d4
            (
            f4
            e4
            )
            d4
        }
        """
    )
    repeater = auxjad.Repeater(container,
                               reposition_clefs=False,
                               reposition_dynamics=False,
                               reposition_slurs=False,
                               )
    notes = repeater(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            \clef "bass"
            f4
            \pp
            (
            e4
            )
            d4
            (
            \clef "bass"
            f4
            \pp
            (
            e4
            )
            d4
            (
            \clef "bass"
            f4
            \pp
            (
            e4
            )
            d4
            (
        }
        """
    )


def test_Repeater_10():
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4 \time 2/4 f'4 g'4")
    repeater = auxjad.Repeater(container,
                               omit_time_signatures=False,
                               force_identical_time_signatures=False,
                               reposition_clefs=True,
                               reposition_dynamics=True,
                               reposition_slurs=True,
                               )
    assert not repeater.omit_time_signatures
    assert not repeater.force_identical_time_signatures
    assert repeater.reposition_clefs
    assert repeater.reposition_dynamics
    assert repeater.reposition_slurs
    repeater.omit_time_signatures = True
    repeater.force_identical_time_signatures = True
    repeater.reposition_clefs = False
    repeater.reposition_dynamics = False
    repeater.reposition_slurs = False
    assert repeater.omit_time_signatures
    assert repeater.force_identical_time_signatures
    assert not repeater.reposition_clefs
    assert not repeater.reposition_dynamics
    assert not repeater.reposition_slurs


def test_Repeater_11():
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4")
    repeater = auxjad.Repeater(container)
    staff = abjad.Staff()
    for window in repeater:
        staff.append(window)
        if abjad.get.duration(staff) == abjad.Duration((9, 4)):
            break
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            \time 3/4
            c'4
            d'4
            e'4
            \time 3/4
            c'4
            d'4
            e'4
        }
        """
    )
    auxjad.mutate.remove_repeated_time_signatures(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
            c'4
            d'4
            e'4
            c'4
            d'4
            e'4
        }
        """
    )


def test_Repeater_12():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    repeater = auxjad.Repeater(container)
    notes = repeater(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'4
            e'4
            f'4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    repeater.contents = abjad.Container(r"c'16 d'16 e'16 f'16 g'2.")
    notes = repeater(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'16
            d'16
            e'16
            f'16
            g'2.
            c'16
            d'16
            e'16
            f'16
            g'2.
        }
        """
    )


def test_Repeater_13():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    repeater = auxjad.Repeater(container)
    notes = repeater.output_n(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'4
            e'4
            f'4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_Repeater_14():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    repeater = auxjad.Repeater(container)
    notes = repeater()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_Repeater_15():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    repeater = auxjad.Repeater(container)
    assert isinstance(repeater(), abjad.Selection)
    tuplet = abjad.Tuplet('3:2', r"c'2 d'2 e'2")
    repeater = auxjad.Repeater(tuplet)
    assert isinstance(repeater(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    repeater = auxjad.Repeater(voice)
    assert isinstance(repeater(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    repeater = auxjad.Repeater(staff)
    assert isinstance(repeater(), abjad.Selection)
    score = abjad.Score([abjad.Staff(r"c'4 d'4 e'4 f'4")])
    repeater = auxjad.Repeater(score)
    assert isinstance(repeater(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    staff = abjad.Staff([voice])
    repeater = auxjad.Repeater(staff)
    assert isinstance(repeater(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    score = abjad.Score([staff])
    repeater = auxjad.Repeater(score)
    assert isinstance(repeater(), abjad.Selection)

    voice1 = abjad.Voice(r"c'4 d'4 e'4 f'4")
    voice2 = abjad.Voice(r"g2 f2")
    staff = abjad.Staff([voice1, voice2], simultaneous=True)
    with pytest.raises(ValueError):
        repeater = auxjad.Repeater(staff)  # noqa: F841

    staff1 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    staff2 = abjad.Staff(r"g2 f2")
    score = abjad.Score([staff1, staff2])
    with pytest.raises(ValueError):
        repeater = auxjad.Repeater(score)  # noqa: F841



import abjad
import pytest

import auxjad


def test_auto_rewrite_meter_01():
    staff = abjad.Staff(
        r"c'16 d'8 e'16 f'8 g'4 a'4 b'8 "
        r"c'16 d'4. e'16 f'8 g'4 a'16 b'16"
    )
    auxjad.mutate.auto_rewrite_meter(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'16
            d'8
            e'16
            f'8
            g'8
            ~
            g'8
            a'4
            b'8
            c'16
            d'8.
            ~
            d'8.
            e'16
            f'8
            g'4
            a'16
            b'16
        }
        """
    )


def test_auto_rewrite_meter_02():
    staff = abjad.Staff(
        r"c'16 d'8 e'16 f'8 g'4 a'4 b'8 "
        r"\time 6/8 b'4 c''4 r4 "
    )
    auxjad.mutate.auto_rewrite_meter(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'16
            d'8
            e'16
            f'8
            g'8
            ~
            g'8
            a'4
            b'8
            \time 6/8
            b'4
            c''8
            ~
            c''8
            r4
        }
        """
    )


def test_auto_rewrite_meter_03():
    staff = abjad.Staff(
        r"c'16 d'8 e'16 f'8 g'4 a'4 b'8 "
        r"c'16 d'4. e'16 f'8 g'4 a'16 b'16"
    )
    abjad.mutate.auto_rewrite_meter(staff, prettify_rewrite_meter=False)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'16
            d'16
            ~
            d'16
            e'16
            f'8
            g'8
            ~
            g'8
            a'8
            ~
            a'8
            b'8
            c'16
            d'8.
            ~
            d'8.
            e'16
            f'8
            g'8
            ~
            g'8
            a'16
            b'16
        }
        """
    )


def test_auto_rewrite_meter_04():
    staff = abjad.Staff(
        r"\time 7/4 c'8 d'4 e'4 f'4 g'4 a'4 b'4 c''8 "
        r"\time 5/4 d''8 e''4 f''4 g''4 a''4 b''8"
    )
    auxjad.mutate.auto_rewrite_meter(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 7/4
            c'8
            d'4
            e'4
            f'8
            ~
            f'8
            g'4
            a'8
            ~
            a'8
            b'4
            c''8
            \time 5/4
            d''8
            e''4
            f''4
            g''8
            ~
            g''8
            a''4
            b''8
        }
        """
    )
    staff = abjad.Staff(
        r"\time 7/4 c'8 d'4 e'4 f'4 g'4 a'4 b'4 c''8 "
        r"\time 5/4 d''8 e''4 f''4 g''4 a''4 b''8"
    )
    meter_list = [abjad.Meter((7, 4), increase_monotonic=True),
                  abjad.Meter((5, 4), increase_monotonic=True),
                  ]
    auxjad.mutate.auto_rewrite_meter(staff, meter_list=meter_list)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 7/4
            c'8
            d'4
            e'8
            ~
            e'8
            f'4
            g'8
            ~
            g'8
            a'4
            b'4
            c''8
            \time 5/4
            d''8
            e''4
            f''8
            ~
            f''8
            g''4
            a''4
            b''8
        }
        """
    )


def test_auto_rewrite_meter_06():
    staff = abjad.Staff(
        r"\time 3/4 c'8 d'4 e'4 f'8 "
        r"\time 5/8 g'4 a'4 r8 "
        r"\time 6/8 b'4 c''4 r4 "
        r"\time 4/4 d''8 e''4 f''8 g''16 a''4 r8."
    )
    abjad.mutate.auto_rewrite_meter(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'8
            d'4
            e'4
            f'8
            \time 5/8
            g'4
            a'8
            ~
            a'8
            r8
            \time 6/8
            b'4
            c''8
            ~
            c''8
            r4
            \time 4/4
            d''8
            e''4
            f''8
            g''16
            a''8.
            ~
            a''16
            r8.
        }
        """
    )


def test_auto_rewrite_meter_07():
    staff = abjad.Staff(r"c'16 d'8 e'16 f'8 g'4 a'4 b'8")
    with pytest.raises(TypeError):
        auxjad.mutate.auto_rewrite_meter(staff[:])


def test_auto_rewrite_meter_08():
    staff = abjad.Staff(
        r"\times 2/3 {c'4 ~ c'8} \times 2/3 {d'8 r4} "
        r"\times 2/3 {r8 r8 r8} \times 2/3 {<e' g'>8 ~ <e' g'>4}"
    )
    abjad.mutate.auto_rewrite_meter(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \times 2/3
            {
                d'8
                r4
            }
            r4
            <e' g'>4
        }
        """
    )


def test_auto_rewrite_meter_09():
    staff = abjad.Staff(
        r"\times 2/3 {c'4 ~ c'8} \times 2/3 {d'8 r4} "
        r"\times 2/3 {r8 r8 r8} \times 2/3 {<e' g'>8 ~ <e' g'>4}"
    )
    abjad.mutate.auto_rewrite_meter(staff, extract_trivial_tuplets=False)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                c'4.
            }
            \times 2/3
            {
                d'8
                r4
            }
            \times 2/3
            {
                r4.
            }
            \times 2/3
            {
                <e' g'>4.
            }
        }
        """
    )


def test_auto_rewrite_meter_10():
    staff = abjad.Staff(
        r"\times 2/3 {c'2 d'1}"
        r"\times 2/3 {e'2} \times 2/3 {f'1}"
    )
    abjad.mutate.auto_rewrite_meter(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                c'2
                d'1
            }
            \times 2/3
            {
                e'2
                f'1
            }
        }
        """
    )
    staff = abjad.Staff(
        r"\times 2/3 {c'2 d'1}"
        r"\times 2/3 {e'2} \times 2/3 {f'1}"
    )
    abjad.mutate.auto_rewrite_meter(staff, merge_partial_tuplets=False)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                c'2
                d'1
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3
            {
                e'2
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3
            {
                f'1
            }
        }
        """
    )


def test_auto_rewrite_meter_11():
    staff = abjad.Staff(r"\time 4/4 c'4. d'4. e'4 f'8 g'4 a'4 b'4.")
    meter = abjad.Meter((4, 4))
    for measure in abjad.select(staff[:]).group_by_measure():
        abjad.Meter.rewrite_meter(measure, meter, boundary_depth=1)
    for measure in abjad.select(staff[:]).group_by_measure():
        auxjad.mutate.prettify_rewrite_meter(measure, meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            ~
            c'8
            d'8
            ~
            d'4
            e'4
            f'8
            g'4
            a'8
            ~
            a'8
            b'8
            ~
            b'4
        }
        """
    )
    staff = abjad.Staff(r"\time 4/4 c'4. d'4. e'4 f'8 g'4 a'4 b'4.")
    auxjad.mutate.auto_rewrite_meter(staff, boundary_depth=1)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            ~
            c'8
            d'8
            ~
            d'4
            e'4
            f'8
            g'8
            ~
            g'8
            a'8
            ~
            a'8
            b'8
            ~
            b'4
        }
        """
    )


def test_auto_rewrite_meter_12():
    staff = abjad.Staff(r"c'16 d'8 e'16 f'8 g'4 a'4 b'8")
    abjad.mutate.auto_rewrite_meter(staff)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'16
            d'8
            e'16
            f'8
            g'8
            ~
            g'8
            a'4
            b'8
        }
        """
    )



import random

import pytest

import auxjad


def test_CartographySelector_01():
    random.seed(41298)
    selector = auxjad.CartographySelector([0, 1, 2, 3, 4])
    assert selector.contents == [0, 1, 2, 3, 4]
    assert format(selector) == '[0, 1, 2, 3, 4]'
    assert selector.weights == [1.0, 0.75, 0.5625, 0.421875, 0.31640625]
    assert selector.previous_index is None
    assert selector.previous_result is None
    assert len(selector) == 5
    result = ''
    for _ in range(30):
        result += str(selector())
    assert result == '203001402200011111101400310140'
    assert selector.previous_index == 0
    assert selector.previous_result == 0
    with pytest.raises(AttributeError):
        selector.previous_index = 3
        selector.previous_result = 7


def test_CartographySelector_02():
    selector = auxjad.CartographySelector([0, 1, 2, 3, 4], decay_rate=0.5)
    assert selector.weights == [1.0, 0.5, 0.25, 0.125, 0.0625]


def test_CartographySelector_03():
    selector = auxjad.CartographySelector([0, 1, 2, 3, 4])
    selector.drop_first_and_append(5)
    assert selector.contents == [1, 2, 3, 4, 5]
    selector.drop_first_and_append(42)
    assert selector.contents == [2, 3, 4, 5, 42]


def test_CartographySelector_04():
    selector = auxjad.CartographySelector([10, 7, 14, 31, 98])
    selector.drop_n_and_append(100, 2)
    assert selector.contents == [10, 7, 31, 98, 100]


def test_CartographySelector_05():
    selector = auxjad.CartographySelector([0, 1, 2, 3, 4])
    selector.drop_last_and_prepend(-1)
    assert selector.contents == [-1, 0, 1, 2, 3]
    selector.drop_last_and_prepend(71)
    assert selector.contents == [71, -1, 0, 1, 2]


def test_CartographySelector_06():
    selector = auxjad.CartographySelector([0, 1, 2, 3, 4])
    selector.rotate()
    assert selector.contents == [1, 2, 3, 4, 0]
    selector.rotate(anticlockwise=True)
    assert selector.contents == [0, 1, 2, 3, 4]
    selector.rotate(anticlockwise=True)
    assert selector.contents == [4, 0, 1, 2, 3]


def test_CartographySelector_07():
    random.seed(15424)
    selector = auxjad.CartographySelector([0, 1, 2, 3, 4])
    selector.shuffle()
    assert selector.contents == [1, 4, 3, 0, 2]


def test_CartographySelector_08():
    selector = auxjad.CartographySelector([0, 1, 2, 3, 4], decay_rate=0.5)
    assert len(selector) == 5
    selector.contents = [0, 1, 2, 3, 4]
    assert selector.weights == [1.0, 0.5, 0.25, 0.125, 0.0625]

    selector.contents = [10, 7, 14, 31, 98, 47, 32]
    assert len(selector) == 7
    assert selector.contents == [10, 7, 14, 31, 98, 47, 32]
    assert selector.weights == [1.0,
                                0.5,
                                0.25,
                                0.125,
                                0.0625,
                                0.03125,
                                0.015625,
                                ]


def test_CartographySelector_09():
    selector = auxjad.CartographySelector([10, 7, 14, 31, 98])
    selector[2] = 100
    assert selector.contents == [10, 7, 100, 31, 98]


def test_CartographySelector_10():
    random.seed(83552)
    selector = auxjad.CartographySelector([0, 1, 2, 3, 4])
    selector.decay_rate = 0.2
    assert selector.weights == [1.0,
                                0.2,
                                0.04000000000000001,
                                0.008000000000000002,
                                0.0016000000000000003,
                                ]
    result = ''
    for _ in range(30):
        result += str(selector())
    assert result == '000001002100000201001030000100'


def test_CartographySelector_11():
    random.seed(19844)
    selector = auxjad.CartographySelector([10, 7, 14, 31, 98])
    assert selector[1] == 7
    assert selector[1:4] == [7, 14, 31]
    assert selector[:] == [10, 7, 14, 31, 98]
    assert selector() == 31
    n = selector.previous_index
    assert n == 3
    assert selector[n] == 31
    del selector[2:4]
    assert selector.contents == [10, 7, 98]


def test_CartographySelector_12():
    random.seed(98743)
    selector = auxjad.CartographySelector([0, 1, 2, 3, 4])
    result = ''
    for _ in range(30):
        result += str(selector())
    assert result == '210431340000344203001220034203'
    result = ''
    for _ in range(30):
        result += str(selector(no_repeat=True))
    assert result == '210421021020304024230120241202'


def test_CartographySelector_13():
    selector = auxjad.CartographySelector([0, 1, 2, 3, 4])
    selector.mirror_swap(0)
    assert selector.contents == [4, 1, 2, 3, 0]
    selector.mirror_swap(0)
    assert selector.contents == [0, 1, 2, 3, 4]
    selector.mirror_swap(3)
    assert selector.contents == [0, 3, 2, 1, 4]
    selector.mirror_swap(2)
    assert selector.contents == [0, 3, 2, 1, 4]


def test_CartographySelector_14():
    random.seed(90129)
    selector = auxjad.CartographySelector([0, 1, 2, 3, 4])
    selector.mirror_random_swap()
    assert selector.contents == [4, 1, 2, 3, 0]
    selector.mirror_random_swap()
    assert selector.contents == [4, 3, 2, 1, 0]
    selector.mirror_random_swap()
    assert selector.contents == [4, 1, 2, 3, 0]


def test_CartographySelector_15():
    random.seed(12387)
    selector = auxjad.CartographySelector(['A', 'B', 'C', 'D', 'E', 'F'])
    result = ''
    result += selector.__next__()
    result += selector.__next__()
    result += selector.__next__()
    result += next(selector)
    result += next(selector)
    result += next(selector)
    assert result == 'CBBEAE'



import random

import abjad

import auxjad


def test_example_of_usage_05():
    random.seed(77124)
    container = abjad.Container([
        auxjad.ArtificialHarmonic(r"<ds' gs'>4"),
        auxjad.ArtificialHarmonic(r"<b e'>8."),
        auxjad.ArtificialHarmonic(r"<g c'>16", is_parenthesized=True),
        abjad.Rest(r"r4"),
        abjad.Chord([2, 8, 9], (1, 8)),
        auxjad.ArtificialHarmonic(r"<d' a'>8", is_parenthesized=True),
    ])
    abjad.mutate.respell_augmented_unisons(container[:])
    shuffler = auxjad.Shuffler(container)
    staff = abjad.Staff()
    notes = shuffler.shuffle_n(4)
    staff.append(notes)
    container = abjad.Container(shuffler.current_window)
    fader = auxjad.Fader(container, mode='out')
    notes = fader.output_all()
    staff.append(notes)
    abjad.mutate.prettify_rewrite_meter(staff[:], abjad.Meter((4, 4)))
    abjad.mutate.remove_repeated_time_signatures(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                g
                \tweak style #'harmonic
                c'
            >16
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                d'
                \tweak style #'harmonic
                a'
            >8
            <d' gs' a'>16
            ~
            <d' gs' a'>16
            r8.
            r16
            <
                b
                \tweak style #'harmonic
                e'
            >8.
            <
                ds'
                \tweak style #'harmonic
                gs'
            >4
            r4
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                g
                \tweak style #'harmonic
                c'
            >16
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                d'
                \tweak style #'harmonic
                a'
            >8
            <d' gs' a'>16
            ~
            <d' gs' a'>16
            <
                b
                \tweak style #'harmonic
                e'
            >8.
            <
                ds'
                \tweak style #'harmonic
                gs'
            >4
            <
                ds'
                \tweak style #'harmonic
                gs'
            >4
            r4
            <
                b
                \tweak style #'harmonic
                e'
            >8.
            <d' gs' a'>16
            ~
            <d' gs' a'>16
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                d'
                \tweak style #'harmonic
                a'
            >8
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                g
                \tweak style #'harmonic
                c'
            >16
            <
                ds'
                \tweak style #'harmonic
                gs'
            >4
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                g
                \tweak style #'harmonic
                c'
            >16
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                d'
                \tweak style #'harmonic
                a'
            >8
            <
                b
                \tweak style #'harmonic
                e'
            >16
            ~
            <
                b
                \tweak style #'harmonic
                e'
            >8
            <d' gs' a'>8
            r4
            <
                ds'
                \tweak style #'harmonic
                gs'
            >4
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                g
                \tweak style #'harmonic
                c'
            >16
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                d'
                \tweak style #'harmonic
                a'
            >8
            <
                b
                \tweak style #'harmonic
                e'
            >16
            ~
            <
                b
                \tweak style #'harmonic
                e'
            >8
            <d' gs' a'>8
            r4
            <
                ds'
                \tweak style #'harmonic
                gs'
            >4
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                g
                \tweak style #'harmonic
                c'
            >16
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                d'
                \tweak style #'harmonic
                a'
            >8
            r16
            r8
            <d' gs' a'>8
            r4
            <
                ds'
                \tweak style #'harmonic
                gs'
            >4
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                g
                \tweak style #'harmonic
                c'
            >16
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                d'
                \tweak style #'harmonic
                a'
            >8
            r16
            r8
            <d' gs'>8
            r4
            <
                ds'
                \tweak style #'harmonic
                gs'
            >4
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                g
                \tweak style #'harmonic
                c'
            >16
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                d'
                \tweak style #'harmonic
                a'
            >8
            r16
            r8
            gs'8
            r4
            <
                ds'
                \tweak style #'harmonic
                gs'
            >4
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                g
                \tweak style #'harmonic
                c'
            >16
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                d'
                \tweak style #'harmonic
                a'
            >8
            r16
            r2
            r4
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                g
                \tweak style #'harmonic
                c'
            >16
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                d'
                \tweak style #'harmonic
                a'
            >8
            r16
            r2
            r4
            <
                \parenthesize
                \tweak ParenthesesItem.font-size -4
                g
                \tweak style #'harmonic
                c'
            >16
            r8.
            r2
            R1
        }
        """
    )



import abjad

import auxjad


def test_TimeSignature_01():
    time_signature = auxjad.TimeSignature((4, 8))
    time_signature.simplify_ratio()
    assert time_signature.pair == (2, 4)


def test_TimeSignature_02():
    time_signature = auxjad.TimeSignature((7, 8))
    time_signature.simplify_ratio()
    assert time_signature.pair == (7, 8)


def test_TimeSignature_03():
    time_signature = auxjad.TimeSignature((10, 16))
    time_signature.simplify_ratio()
    assert time_signature.pair == (5, 8)


def test_TimeSignature_04():
    time_signature = auxjad.TimeSignature((4, 8))
    time_signature.simplify_ratio(min_denominator=8)
    assert time_signature.pair == (4, 8)


def test_TimeSignature_05():
    time_signature = auxjad.TimeSignature((4, 8))
    time_signature.simplify_ratio(min_denominator=2)
    assert time_signature.pair == (1, 2)


def test_TimeSignature_06():
    time_signature = auxjad.TimeSignature((1, 1))
    time_signature.simplify_ratio()
    assert time_signature.pair == (4, 4)


def test_TimeSignature_07():
    time_signature = auxjad.TimeSignature((1, 1))
    time_signature.simplify_ratio(min_denominator=1)
    assert time_signature.pair == (1, 1)


def test_TimeSignature_08():
    time_signature = auxjad.TimeSignature((8, 8))
    time_signature.simplify_ratio()
    assert time_signature.pair == (4, 4)


def test_TimeSignature_09():
    assert (auxjad.TimeSignature.simplify_ratio
            is abjad.TimeSignature.simplify_ratio)



import random

import abjad

import auxjad


def test_example_of_usage_04():
    random.seed(98611)
    container = abjad.Container(r"c'4 ~ c'16 r8 c'16 c'4 c'16 r8.")
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches=[0, 1, 2, 3, 4, 5, 6],
                                        )
    staff = abjad.Staff()
    notes = randomiser.output_n(3)
    staff.append(notes)
    randomiser.pitches = [13, 14, 16, 17, 21]
    notes = randomiser.output_n(3)
    staff.append(notes)
    randomiser.weights = [6, 3, 2, 1, 1]
    notes = randomiser.output_n(3)
    staff.append(notes)
    hocketer = auxjad.Hocketer(staff,
                               n_voices=3,
                               )
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                f'4
                ~
                f'16
                r8.
                r4
                cs'16
                r8.
                R1
                R1
                f''4
                ~
                f''16
                r16
                r16
                f''16
                r2
                R1
                a''4
                ~
                a''16
                r8.
                e''4
                r4
                r2.
                d''16
                r8.
                r4..
                cs''16
                r2
                r4..
                cs''16
                r4
                cs''16
                r8.
            }
            \new Staff
            {
                r2
                d'4
                r4
                fs'4
                ~
                fs'16
                r8.
                r2
                c'4
                ~
                c'16
                r16
                r16
                ef'16
                r4
                cs'16
                r8.
                r2.
                f''16
                r8.
                f''4
                ~
                f''16
                r16
                r16
                a''16
                e''4
                f''16
                r8.
                r4..
                cs''16
                r2
                r4..
                d''16
                r2
                d''4
                ~
                d''16
                r8.
                r4
                a''16
                r8.
                R1
            }
            \new Staff
            {
                r4..
                fs'16
                r2
                r4..
                cs'16
                ef'4
                e'16
                r8.
                r2
                c'4
                r4
                r2
                cs''4
                r4
                R1
                r2.
                f''16
                r8.
                cs''4
                ~
                cs''16
                r8.
                d''4
                r4
                r2
                d''4
                r4
                d''4
                ~
                d''16
                r8.
                d''4
                r4
            }
        >>
        """
    )



import abjad

import auxjad


def test_extract_trivial_tuplets_01():
    staff = abjad.Staff(r"\times 2/3 {r4 r2} \times 2/3 {c'8 ~ c'8 ~ c'2}")
    auxjad.mutate.extract_trivial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r2
            c'2
        }
        """
    )


def test_extract_trivial_tuplets_02():
    staff = abjad.Staff(r"\times 4/5 {r2. \times 2/3 {r2 r4}}")
    auxjad.mutate.extract_trivial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r1
        }
        """
    )


def test_extract_trivial_tuplets_03():
    staff = abjad.Staff(r"\times 4/5 {c'2. ~ \times 2/3 {c'2 ~ c'4}}")
    auxjad.mutate.extract_trivial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
        }
        """
    )


def test_extract_trivial_tuplets_04():
    staff = abjad.Staff(r"\times 2/3 {<c' d'>1 ~ <c' d'>2}")
    auxjad.mutate.extract_trivial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <c' d'>1
        }
        """
    )


def test_extract_trivial_tuplets_05():
    staff = abjad.Staff(
        r"\times 2/3 {r2 r1} \times 4/5 {c'2. \times 2/3 {r2 r4}}"
    )
    auxjad.mutate.extract_trivial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r1
            \times 4/5
            {
                c'2.
                r2
            }
        }
        """
    )


def test_extract_trivial_tuplets_06():
    staff = abjad.Staff(r"\time 3/4 r2. \times 3/2 {r4 r4}")
    auxjad.mutate.extract_trivial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            r2.
            r2.
        }
        """
    )


def test_extract_trivial_tuplets_07():
    staff = abjad.Staff(r"\times 2/3 {r2 r2 r2}")
    abjad.mutate.extract_trivial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r1
        }
        """
    )


def test_extract_trivial_tuplets_08():
    staff = abjad.Staff(
        r"\times 2/3 {c'4} r2 \times 2/3 {d'2}"
        r"\times 2/3 {e'1.}"
    )
    abjad.mutate.extract_trivial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \tweak edge-height #'(0.7 . 0)
            \times 2/3
            {
                c'4
            }
            r2
            \tweak edge-height #'(0.7 . 0)
            \times 2/3
            {
                d'2
            }
            e'1
        }
        """
    )


def test_extract_trivial_tuplets_09():
    staff = abjad.Staff(r"\time 6/4 c'4\f \times 5/6 {g1.\p}")
    abjad.mutate.extract_trivial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 6/4
            c'4
            \f
            g1
            \p
            ~
            g4
        }
        """
    )


def test_extract_trivial_tuplets_10():
    staff = abjad.Staff(
        r"\times 2/3 { r2 c'4 ~ } \times 2/3 { c'2 ~ c'4 ~ } c'1"
    )
    abjad.mutate.extract_trivial_tuplets(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                r2
                c'4
                ~
            }
            c'2
            ~
            c'1
        }
        """
    )
